/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_CANVASUID_H
#define CROW_CANVASUID_H

#include "widgets.h"
#include "editors.h"

namespace Crow {

class UIDefinitionCanvasEditorWidget : public EditorWidget {

public:
	UIDefinitionCanvasEditorWidget();

	PGtkActionGroup getActions();
	PGtkUIManager getManager();
	Polytree * getTreeView() { return &treeView; }

protected:
	PGtkActionGroup createActionGroup();
	PGtkUIManager createUIManager(PGtkActionGroup actionGroup);

private:
	Gtk::HBox box;
	Gtk::ScrolledWindow scroll;
	Polytree treeView;

	PGtkUIManager uiManager;
	PGtkActionGroup actionGroup;

};

typedef std::vector<int> UiPath;

struct UIElement : public Object {
	bool remove;
	UiPath path;
	std::string element;
	std::string name;
	std::string action;
	bool expanded;
	UIElement(
		const UiPath & path,
		string_cr element,
		string_cr name,
		string_cr action,
		bool expanded=true
	);
	bool equals(PObject elem);
};

class UIDefinitionCanvasEditor : public CanvasEditor, public TCreateEditorWidget<UIDefinitionCanvasEditorWidget>, public Parser {

public:
	UIDefinitionCanvasEditor();

	void created();
	void load(bool reload);
	void storeSettings();
	void deleted();

protected:

	PUIElement getSelected();
	PUIElement getParent(PUIElement elem);

	void onSelectionChanged();
	void updateActions();
	bool anchestry(PUIElement obj, string_cr element);
	void updateElementActions(PUIElement parent);
	void popupNewChild(PUIElement parent);
	void onActionNew();
	void onActionAdd();
	void onActionRemove();
	void onActionMoveUp();
	void onActionMoveDown();
	void onActionElement(const char * element);

	std::vector<PUIElement> cut(const UiPath & path, bool siblings);
	void paste(const std::vector<PUIElement> & vec, int shift);
	void updateModel(const std::vector<int> & path);

	std::string toString();
	void fromString(string_cr);

	void onStartElement(ustring_cr element_name, const Glib::Markup::Parser::AttributeMap & attributes);
	void onEndElement(ustring_cr element_name);

	void updateTree(bool expand=true);
	void setRowValues(PPolyelem elem);
	bool validator(ustring_cr oldtext, ustring_cr newtext);
	void onSetName(PolycellInput * polycell, PUIElement elem);
	void onSetAction(PolycellInput * polycell, PUIElement elem);
	void onPopupActions(PUIElement elem);
	void onActionSelect(PUIElement elem, string_cr action);
	void onRowCollapsed(const Gtk::TreeModel::iterator &, const Gtk::TreeModel::Path &);
	void onRowExpanded(const Gtk::TreeModel::iterator &, const Gtk::TreeModel::Path &);

	PUIElement getElement(const UiPath & path);
	void setElement(const UiPath & path, PUIElement element);

	void dump(string_cr str);

private:
	typedef std::map<UiPath, PUIElement> Elements;
	Elements elements;
	UiPath pstack;
	UiPath actionElement;
	bool lock;

};

}

#endif
