/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef CROW_CROW_H
#define CROW_CROW_H

#include <gtkmm.h>

#define CROW_VERSION (2*1000000 + 17*1000 + 1)

#ifdef _WIN32
	#ifdef CROW_LIBRARY
		#define CROW_API __declspec(dllexport)
	#else
		#define CROW_API __declspec(dllimport)
	#endif
#else
	#define CROW_API __attribute__ ((visibility("default")))
#endif

namespace Crow {

typedef gint64 CrowActions;
const CrowActions CROW_ACTION_SAVE          = 1LL <<  0;
const CrowActions CROW_ACTION_SHOULD_SAVE   = 1LL <<  1;
const CrowActions CROW_ACTION_UNDO          = 1LL <<  2;
const CrowActions CROW_ACTION_REDO          = 1LL <<  3;
const CrowActions CROW_ACTION_CUT           = 1LL <<  4;
const CrowActions CROW_ACTION_COPY          = 1LL <<  5;
const CrowActions CROW_ACTION_PASTE         = 1LL <<  6;
const CrowActions CROW_ACTION_PASTE_TO_ROOT = 1LL <<  7;
const CrowActions CROW_ACTION_DELETE        = 1LL <<  8;
const CrowActions CROW_ACTION_ABOUT         = 1LL <<  9;
const CrowActions CROW_ACTION_PREVIEW       = 1LL << 10;
const CrowActions CROW_ACTION_DEFAULT_SIZE  = 1LL << 11;
const CrowActions CROW_ACTION_SHIFT_LEFT    = 1LL << 12;
const CrowActions CROW_ACTION_SHIFT_RIGHT   = 1LL << 13;
const CrowActions CROW_ACTION_TRANSLATABLES = 1LL << 14;
const CrowActions CROW_ACTION_DEFAULT       = 1LL << 15;
const CrowActions CROW_ACTION_ADD           = 1LL << 16;
const CrowActions CROW_ACTION_REMOVE        = 1LL << 17;
const CrowActions CROW_ACTION_MOVE_UP       = 1LL << 18;
const CrowActions CROW_ACTION_MOVE_DOWN     = 1LL << 19;

const CrowActions CROW_ACTIONS_SAVE = CROW_ACTION_SAVE|CROW_ACTION_SHOULD_SAVE;
const CrowActions CROW_ACTIONS_BASIC =
	CROW_ACTION_UNDO|CROW_ACTION_REDO|CROW_ACTION_CUT|CROW_ACTION_COPY|
	CROW_ACTION_PASTE|CROW_ACTION_PASTE_TO_ROOT|CROW_ACTION_DELETE|
	CROW_ACTION_ABOUT|CROW_ACTION_PREVIEW|CROW_ACTION_DEFAULT_SIZE|
	CROW_ACTION_SHIFT_LEFT|CROW_ACTION_SHIFT_RIGHT|CROW_ACTION_TRANSLATABLES;

const CrowActions CROW_ACTIONS_EXPLORER =
	CROW_ACTION_DEFAULT|CROW_ACTION_ADD|CROW_ACTION_REMOVE|
	CROW_ACTION_MOVE_UP|CROW_ACTION_MOVE_DOWN;

const CrowActions CROW_ACTIONS_ALL = CROW_ACTIONS_SAVE|CROW_ACTIONS_BASIC|CROW_ACTIONS_EXPLORER;

class Designer;
typedef Glib::RefPtr<Designer> PDesigner;

typedef std::map<std::string, Glib::ustring> Settings;

class Designer : virtual public Glib::ObjectBase {

public:
	virtual void apply_settings(const Settings & settings) = 0;
	virtual void store_settings(Settings * settings) = 0;
	virtual void clear() = 0;
	virtual void load(const Glib::ustring & data) = 0;
	virtual Glib::ustring save() = 0;
	virtual Glib::ustring save_header() = 0;
	virtual void saved() = 0;
	virtual Gtk::Widget * get_widget() = 0;
	virtual CrowActions get_enabled_actions() = 0;
	virtual void run_action(CrowActions action) = 0;

	typedef sigc::signal<void, CrowActions> SignalActionsUpdated;
	typedef sigc::signal<void, Glib::ustring> SignalStatusUpdated;
	typedef sigc::signal<void, Glib::ustring> SignalInfoUpdated;
	typedef sigc::signal<void, Glib::ustring> SignalShowUrl;
	typedef sigc::signal<void> SignalModalEnter;
	typedef sigc::signal<void> SignalModalLeave;

	virtual sigc::connection connect_actions_updated(SignalActionsUpdated::slot_type) = 0;
	virtual sigc::connection connect_status_updated(SignalStatusUpdated::slot_type) = 0;
	virtual sigc::connection connect_info_updated(SignalInfoUpdated::slot_type) = 0;
	virtual sigc::connection connect_show_url(SignalShowUrl::slot_type) = 0;
	virtual sigc::connection connect_modal_enter(SignalModalEnter::slot_type) = 0;
	virtual sigc::connection connect_modal_leave(SignalModalLeave::slot_type) = 0;

};

CROW_API PDesigner CreateDesigner();
CROW_API int GetCrowVersion();

class Enter {

public:
	CROW_API Enter(const char * msg = NULL);
	CROW_API ~Enter();

};

}

#endif
