/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef CROW_CROW_IMPL_H
#define CROW_CROW_IMPL_H

#include "crow.h"
#include "util.h"

namespace Crow {

class CrowInit {

friend class Crow::Object;
friend class Palette;

public:
	static CrowInit * get();

	static void enter(string_cr msg);
	static void leave();

protected:
	static CrowInit * & ptr();

	CrowInit();
	~CrowInit();

	void init();

	void snapshot(string_cr msg);
	void check();

	Palette * getPalette() { return palette; }

private:
	Gui::Enter guiEnter;
	int refcounter;
	Palette * palette;

	#ifdef CROW_SANDBOX
		typedef std::set<Crow::Object *> Set;
		Set objects;
		std::list<Set> snapshots;
	#else
		int objects;
		std::list<int> snapshots;
	#endif

	std::list<std::string> labels;

	void objectCreated(Crow::Object * object);
	void objectDeleted(Crow::Object * object);

};

}

#endif
