/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#include "headers.h"
#include "forward.h"
#include "views1.h"

#include "palette.h"
#include "controller.h"

namespace Crow {

void RegisterViews1() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<RowChildView>("CrowRowChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TWidgetEntry<GtkAlignmentView>("GtkAlignment", GTK_TYPE_ALIGNMENT, TAB_BINS));

	palette->takeEntry(new EnumEntry("GtkShadowType")).
		addValue("GTK_SHADOW_NONE", Gtk::SHADOW_NONE).
		addValue("GTK_SHADOW_IN", Gtk::SHADOW_IN).
		addValue("GTK_SHADOW_OUT", Gtk::SHADOW_OUT).
		addValue("GTK_SHADOW_ETCHED_IN", Gtk::SHADOW_ETCHED_IN).
		addValue("GTK_SHADOW_ETCHED_OUT", Gtk::SHADOW_ETCHED_OUT);

	palette->takeEntry(new TWidgetEntry<GtkFrameView>("GtkFrame", GTK_TYPE_FRAME, TAB_BINS));

	palette->takeEntry(new TWidgetEntry<GtkAspectFrameView>("GtkAspectFrame", GTK_TYPE_ASPECT_FRAME, TAB_BINS));

	palette->takeEntry(new TWidgetEntry<GtkExpanderView>("GtkExpander", GTK_TYPE_EXPANDER, TAB_BINS));

	palette->takeEntry(new EnumEntry("GtkPolicyType")).
		addValue("GTK_POLICY_ALWAYS", Gtk::POLICY_ALWAYS).
		addValue("GTK_POLICY_AUTOMATIC", Gtk::POLICY_AUTOMATIC).
		addValue("GTK_POLICY_NEVER", Gtk::POLICY_NEVER);

	palette->takeEntry(new EnumEntry("GtkCornerType")).
		addValue("GTK_CORNER_TOP_LEFT", Gtk::CORNER_TOP_LEFT).
		addValue("GTK_CORNER_BOTTOM_LEFT", Gtk::CORNER_BOTTOM_LEFT).
		addValue("GTK_CORNER_TOP_RIGHT", Gtk::CORNER_TOP_RIGHT).
		addValue("GTK_CORNER_BOTTOM_RIGHT", Gtk::CORNER_BOTTOM_RIGHT);

	palette->takeEntry(new TWidgetEntry<GtkScrolledWindowView>("GtkScrolledWindow", GTK_TYPE_SCROLLED_WINDOW, TAB_BINS));

	palette->takeEntry(new EnumEntry("GtkPositionType")).
		addValue("GTK_POS_LEFT", Gtk::POS_LEFT).
		addValue("GTK_POS_RIGHT", Gtk::POS_RIGHT).
		addValue("GTK_POS_TOP", Gtk::POS_TOP).
		addValue("GTK_POS_BOTTOM", Gtk::POS_BOTTOM);

	palette->takeEntry(new TWidgetEntry<GtkHandleBoxView>("GtkHandleBox", GTK_TYPE_HANDLE_BOX, TAB_BINS));

	palette->takeEntry(new TWidgetEntry<GtkViewportView>("GtkViewport", GTK_TYPE_VIEWPORT, TAB_BINS));

	// -----------------------------------

	palette->takeEntry(new EnumEntry("CrowPackOptions")).
		addValue("CROW_PACK_SHRINK", Gtk::PACK_SHRINK).
		addValue("CROW_PACK_EXPAND_PADDING", Gtk::PACK_EXPAND_PADDING).
		addValue("CROW_PACK_EXPAND_WIDGET", Gtk::PACK_EXPAND_WIDGET);

	palette->takeEntry(new TObjectEntry<BoxChildView>("CrowBoxChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TWidgetEntry<GtkHBoxView>("GtkHBox", GTK_TYPE_HBOX, TAB_LAYOUTS));

	palette->takeEntry(new TWidgetEntry<GtkVBoxView>("GtkVBox", GTK_TYPE_VBOX, TAB_LAYOUTS));

	palette->takeEntry(new TObjectEntry<ButtonBoxChildView>("CrowButtonBoxChild")).
		setIsA("CrowChild");

	palette->takeEntry(new EnumEntry("GtkButtonBoxStyle")).
		addValue("GTK_BUTTONBOX_DEFAULT_STYLE", Gtk::BUTTONBOX_DEFAULT_STYLE).
		addValue("GTK_BUTTONBOX_SPREAD", Gtk::BUTTONBOX_SPREAD).
		addValue("GTK_BUTTONBOX_EDGE", Gtk::BUTTONBOX_EDGE).
		addValue("GTK_BUTTONBOX_START", Gtk::BUTTONBOX_START).
		addValue("GTK_BUTTONBOX_END", Gtk::BUTTONBOX_END);

	palette->takeEntry(new EnumEntry("GtkAssistantPageType")).
		addValue("GTK_ASSISTANT_PAGE_CONTENT", Gtk::ASSISTANT_PAGE_CONTENT).
		addValue("GTK_ASSISTANT_PAGE_INTRO", Gtk::ASSISTANT_PAGE_INTRO).
		addValue("GTK_ASSISTANT_PAGE_CONFIRM", Gtk::ASSISTANT_PAGE_CONFIRM).
		addValue("GTK_ASSISTANT_PAGE_SUMMARY", Gtk::ASSISTANT_PAGE_SUMMARY).
		addValue("GTK_ASSISTANT_PAGE_PROGRESS", Gtk::ASSISTANT_PAGE_PROGRESS);

	palette->takeEntry(new TWidgetEntry<GtkHButtonBoxView>("GtkHButtonBox", GTK_TYPE_HBUTTON_BOX, TAB_LAYOUTS));

	palette->takeEntry(new TWidgetEntry<GtkVButtonBoxView>("GtkVButtonBox", GTK_TYPE_VBUTTON_BOX, TAB_LAYOUTS));

	palette->takeEntry(new TObjectEntry<PanedChildView>("CrowPanedChild")).
		setIsA("CrowChild");

	palette->takeEntry(new AbstractGtkWidgetEntry("GtkPaned", GTK_TYPE_PANED));

	palette->takeEntry(new TWidgetEntry<GtkHPanedView>("GtkHPaned", GTK_TYPE_HPANED, TAB_LAYOUTS));

	palette->takeEntry(new TWidgetEntry<GtkVPanedView>("GtkVPaned", GTK_TYPE_VPANED, TAB_LAYOUTS));

	palette->takeEntry(new TObjectEntry<NotebookChildView>("CrowNotebookChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TWidgetEntry<GtkNotebookView>("GtkNotebook", GTK_TYPE_NOTEBOOK, TAB_LAYOUTS));

	palette->takeEntry(new TObjectEntry<AssistantChildView>("CrowAssistantChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TObjectEntry<TableChildView>("CrowTableChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TWidgetEntry<GtkTableView>("GtkTable", GTK_TYPE_TABLE, TAB_LAYOUTS));

	// -----------------------------------

	palette->takeEntry(new TObjectEntry<XYChildView>("CrowXYChild")).
		setIsA("CrowChild");

	palette->takeEntry(new TObjectEntry<GtkAdjustmentView>("GtkAdjustment", GTK_TYPE_ADJUSTMENT)).
		setFlags(efInternal);

}

// GtkBinView ------------------------------------------------------------------

GtkBinView::GtkBinView() {
	setChildType("CrowRowChild");
	findProperty("focus-chain")->setFlag(sfNoEditing|sfHidden);
}

void GtkBinView::initDesign() {
	if(isRoot())
		setupAsContainer(true);
}

void GtkBinView::setupContainer() {
	getContainer()->set(Children(), placeholdersSet());
}

void GtkBinView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value), placeholdersSet());
}

// GtkAlignmentView ------------------------------------------------------------

GtkAlignmentView::GtkAlignmentView() {
	addProperty("bottom-padding", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("left-padding", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("right-padding", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("top-padding", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("xscale", prScalar, "float", CAny::createFloat(1));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("yscale", prScalar, "float", CAny::createFloat(1));
}

// GtkFrameView ----------------------------------------------------------------

GtkFrameView::GtkFrameView() {
	addInertProperty("label-widget-set", prScalar, "bool", CAny::createBool(true)).
		setValueSet(sigc::mem_fun(*this, &GtkFrameView::setLabelWidgetSet));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setValueGet(sigc::mem_fun(*this, &GtkFrameView::getLabel)).
		setValueSet(sigc::mem_fun(*this, &GtkFrameView::setLabel)).
		setFlag(sfReadOnly | sfTranslatable);
	addProperty("label-widget", prEntity, "GtkWidget").
		setViewConfigure(sigc::mem_fun(*this, &GtkFrameView::configureLabelWidget));
	addProperty("label-xalign", prScalar, "float", CAny::createFloat(0.0f));
	addProperty("label-yalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_ETCHED_IN));
}

void GtkFrameView::setLabelWidgetSet(Property * property, Any value) {
	property->setInert(value);
	bool set= value->getBool();
	Glib::RefPtr<Gtk::Frame> widget= getObject();
	widget->unset_label();
	if(!set) {
		widget->set_label((isEditing() || isSeeking()) ? "text" : "");
		findProperty("label")->clearFlag(sfReadOnly);
		findProperty("label-widget")->setFlag(sfReadOnly);
	} else {
		CHECK(isEditing() || isSeeking());
		findProperty("label")->setFlag(sfReadOnly);
		findProperty("label-widget")->clearFlag(sfReadOnly);
	}
	touch("label");
	touch("label-widget");
}

Any GtkFrameView::getLabel(Property * property) {
	bool label= !findProperty("label-widget-set")->getInert()->getBool();
	if(label) {
		Gtk::Label * widget= dynamic_cast<Gtk::Label *>(getObject()->get_label_widget());
		return CAny::createString(widget ? widget->get_label() : "");
	} else
		return CAny::createString("");
}

void GtkFrameView::setLabel(Property * property, Any value) {
	Gtk::Label * label= dynamic_cast<Gtk::Label *>(getObject()->get_label_widget());
	if(label)
		label->set_label(*value->getString());
	else
		getObject()->set_label(*value->getString());
}

void GtkFrameView::configureLabelWidget(Property * property, PEntityView view) {
	if(findProperty("label-widget-set")->getInert()->getBool() == false)
		view->findProperty("label")->setFlag(sfReadOnly);
}

// GtkAspectFrameView ----------------------------------------------------------

GtkAspectFrameView::GtkAspectFrameView() {
	addProperty("obey-child", prScalar, "bool", CAny::createBool(true));
	addProperty("ratio", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
}

Any GtkAspectFrameView::createInstance() {
	Gtk::AspectFrame * widget= new Gtk::AspectFrame();
	widget->unset_label();
	widget->set(0.5f, 0.5f, 0.5f, true);
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

// GtkExpanderView -------------------------------------------------------------

GtkExpanderView::GtkExpanderView() {
	addCanFocusProperty(true);
	addInertProperty("label-widget-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkExpanderView::setLabelWidgetSet));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkExpanderView::getLabel)).
		setValueSet(sigc::mem_fun(*this, &GtkExpanderView::setLabel));
	addProperty("label-widget", prEntity, "GtkWidget").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkExpanderView::configureLabelWidget));
	addProperty("expanded", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkExpanderView::setExpanded));
	addProperty("spacing", prScalar, "int", CAny::createInt(0));
}

void GtkExpanderView::initInstance() {
	GtkBinView::initInstance();
	getObject()->set_label("");
}

void GtkExpanderView::initDesign() {
	GtkBinView::initDesign();
	getObject()->set_label(getDesignLabel());
}

bool GtkExpanderView::isTopChild(PChild child) {
	return isActiveWidget(GetPtr(child->widget));
}

bool GtkExpanderView::isActiveWidget(Gtk::Widget * widget) {
	if(getObject()->get_child() == widget)
		return getObject()->get_expanded();
	CHECK(getObject()->get_label_widget() == widget);
	return true;
}

void GtkExpanderView::setLabelWidgetSet(Property * property, Any value) {
	property->setInert(value);
	bool set= value->getBool();
	Glib::RefPtr<Gtk::Expander> widget= getObject();
	widget->property_label_widget()= NULL;
	if(!set) {
		CHECK(isEditing() || isSeeking());
		widget->set_label("text");
		findProperty("label")->clearFlag(sfReadOnly);
		findProperty("label-widget")->setFlag(sfReadOnly);
	} else {
		findProperty("label")->setFlag(sfReadOnly);
		findProperty("label-widget")->clearFlag(sfReadOnly);
	}
	touch("label");
	touch("label-widget");
}

Any GtkExpanderView::getLabel(Property * property) {
	bool label= !findProperty("label-widget-set")->getInert()->getBool();
	if(label) {
		Gtk::Label * widget= dynamic_cast<Gtk::Label *>(getObject()->get_label_widget());
		return CAny::createString(widget ? widget->get_label() : "");
	} else
		return CAny::createString("");
}

void GtkExpanderView::setLabel(Property * property, Any value) {
	Gtk::Label * label= dynamic_cast<Gtk::Label *>(getObject()->get_label_widget());
	if(label)
		label->set_label(*value->getString());
	else
		getObject()->set_label(*value->getString());
}

void GtkExpanderView::configureLabelWidget(Property * property, PEntityView view) {
	if(findProperty("label-widget-set")->getInert()->getBool() == false)
		view->findProperty("label")->setFlag(sfReadOnly);
}

void GtkExpanderView::setExpanded(Property * property, Any value) {
	Gtk::Expander * widget= GetPtr(getObject());
	if(isEditing() || isSeeking()) {
		PGtkWidget child= MakeRefPtr(widget->get_child());
		BinRemove(widget);
		widget->set_expanded(value->getBool());
		if(child)
			widget->add(*GetPtr(child));
	} else
		widget->set_expanded(value->getBool());
}

// GtkScrolledWindowView -------------------------------------------------------

GtkScrolledWindowView::GtkScrolledWindowView() {
	setInteractive(true);
	addCanFocusProperty(true);
	addProperty("hadjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkScrolledWindowView::configureAdjustment));
	addProperty("vadjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly).
		setViewConfigure(sigc::mem_fun(*this, &GtkScrolledWindowView::configureAdjustment));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_NONE));
	addProperty("hscrollbar-policy", prScalar, "GtkPolicyType", CAny::createEnum("GtkPolicyType", Gtk::POLICY_ALWAYS));
	addProperty("vscrollbar-policy", prScalar, "GtkPolicyType", CAny::createEnum("GtkPolicyType", Gtk::POLICY_ALWAYS));
	addProperty("window-placement-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkScrolledWindowView::setWindowPlacementSet));
	addProperty("window-placement", prScalar, "GtkCornerType", CAny::createEnum("GtkCornerType", Gtk::CORNER_TOP_LEFT)).
		setFlag(sfReadOnly);
	addProperty("auto-viewport", prEntity, "GtkViewport").
		setFlag(sfReadOnly).
		setValueGet(sigc::mem_fun(*this, &GtkScrolledWindowView::getAutoViewport));
}

Slider GtkScrolledWindowView::getSlider(Manipulator manip) {
	Gtk::Adjustment * adj= manip==SliderHor ?
		getObject()->get_hadjustment() : getObject()->get_vadjustment();
	Slider slider;
	slider.manip= manip;
	slider.value= GetSliderValue(adj);
	return slider;
}

void GtkScrolledWindowView::setSlider(Manipulator manip, double value) {
	Gtk::Adjustment * adj= manip==SliderHor ?
		getObject()->get_hadjustment() : getObject()->get_vadjustment();
	SetSliderValue(adj, value);
}

ScrolledWindowContainer * GtkScrolledWindowView::createContainer() {
	return new ScrolledWindowContainer();
}

Rectangle GtkScrolledWindowView::getChildArea() {
	Gtk::Widget * child= getObject()->get_child();
	if(child)
		return getWidgetRect(MakeRefPtr(child));
	else
		return GtkBinView::getChildArea();
}

void GtkScrolledWindowView::configureAdjustment(Property * property, PEntityView ev) {
	GtkAdjustmentView * view= dynamic_cast<GtkAdjustmentView *>(GetPtr(ev));
	view->configureForScrolling();
}

void GtkScrolledWindowView::setWindowPlacementSet(Property * property, Any value) {
	bool set= value->getBool();
	getObject()->set_placement(Gtk::CORNER_TOP_LEFT);
	if(set)
		findProperty("window-placement")->clearFlag(sfReadOnly);
	else
		findProperty("window-placement")->setFlag(sfReadOnly);
	getObject()->set_property("window-placement-set", set);
	touch("window-placement");
}

Any GtkScrolledWindowView::getAutoViewport(Property * property) {
	Gtk::Widget * widget= getObject()->get_child();
	if(widget && GetData(widget, "CrowAutoViewportHint").empty())
		widget= NULL;
	if(widget && getChildren().empty())
		widget= NULL;
	return CAny::createObject(MakeRefPtr(widget));
}

// GtkHandleBoxView ------------------------------------------------------------

GtkHandleBoxView::GtkHandleBoxView() {
	addEventProperties();
	addProperty("handle-position", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_LEFT));
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_ETCHED_OUT));
	addProperty("snap-edge-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkHandleBoxView::setSnapEdgeSet));
	addProperty("snap-edge", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_TOP)).
		setFlag(sfReadOnly);
}

void GtkHandleBoxView::initInstance() {
	GtkBinView::initInstance();
	getObject()->set_shadow_type(Gtk::SHADOW_ETCHED_OUT);
}

void GtkHandleBoxView::setSnapEdgeSet(Property * property, Any value) {
	bool set= value->getBool();
	getObject()->set_snap_edge(Gtk::POS_TOP);
	if(set)
		findProperty("snap-edge")->clearFlag(sfReadOnly);
	else
		findProperty("snap-edge")->setFlag(sfReadOnly);
	getObject()->set_property("snap-edge-set", set);
	touch("snap-edge");
}

// GtkViewportView -------------------------------------------------------------

GtkViewportView::GtkViewportView() {
	addEventProperties();
	addProperty("shadow-type", prScalar, "GtkShadowType", CAny::createEnum("GtkShadowType", Gtk::SHADOW_IN));
}

Any GtkViewportView::createInstance() {
	Gtk::Adjustment * hadj= Gtk::manage(new Gtk::Adjustment(0, 0, 0, 0, 0, 0));
	Gtk::Adjustment * vadj= Gtk::manage(new Gtk::Adjustment(0, 0, 0, 0, 0, 0));
	Gtk::Viewport * widget= new Gtk::Viewport(*hadj, *vadj);
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

// BoxChildView ----------------------------------------------------------------

BoxChildView::BoxChildView() {
	addIndexProperty();
	addProperty("pack", prScalar, "int").
		setFlag(sfNoEditing | sfChildDef).
		setValueGet(sigc::mem_fun(*this, &BoxChildView::getPack)).
		setValueSet(sigc::mem_fun(*this, &BoxChildView::setPack));
	addProperty("options", prScalar, "CrowPackOptions", CAny::createEnum("CrowPackOptions", Gtk::PACK_EXPAND_WIDGET)).
		setValueGet(sigc::mem_fun(*this, &BoxChildView::getOptions)).
		setValueSet(sigc::mem_fun(*this, &BoxChildView::setOptions));
	addProperty("padding", prScalar, "unsigned", CAny::createUnsigned(0)).
		setValueGet(sigc::mem_fun(*this, &BoxChildView::getPadding)).
		setValueSet(sigc::mem_fun(*this, &BoxChildView::setPadding));
}

Any BoxChildView::getPack(Property * property) {
	return CAny::createInt(getObject()->place.b);
}

void BoxChildView::setPack(Property * property, Any value) {
	getObject()->place.b= value->getInt();
}

Any BoxChildView::getOptions(Property * property) {
	return CAny::createEnum("CrowPackOptions", getObject()->options);
}

void BoxChildView::setOptions(Property * property, Any value) {
	getObject()->options= (Gtk::PackOptions)value->getEnum();
}

Any BoxChildView::getPadding(Property * property) {
	return CAny::createUnsigned(getObject()->padding);
}

void BoxChildView::setPadding(Property * property, Any value) {
	getObject()->padding= value->getUnsigned();
}

// GtkBoxView ------------------------------------------------------------------

GtkBoxView::GtkBoxView() {
	setChildType("CrowBoxChild");
	addInertProperty("capacity", prScalar, "CrowPoint", CAny::createPoint(Point(0, 0))).
		setFlag(sfReadOnly).
		setValueSet(sigc::mem_fun(*this, &GtkBoxView::setCapacityProperty));
	moveBefore("capacity", "children");
	addOrientationProperty();
	addProperty("homogeneous", prScalar, "bool", CAny::createBool(false));
	addProperty("spacing", prScalar, "int", CAny::createInt(0));
}

PNode GtkBoxView::createChild(const Place & place, PNode widget) {
	Model * model= getModel();
	PNode child= GtkContainerView::createChild(place, widget);
	model->createScalar(child, "index", CAny::createInt(place.a));
	model->createScalar(child, "pack", CAny::createInt(place.b));
	return child;
}

void GtkBoxView::initDesign() {
	if(isRoot()) {
		setupAsContainer(true);
		getContainer()->set(Children(), 3, 0);
		findProperty("capacity")->setInert(CAny::createPoint(Point(3, 0)));
	}
}

void GtkBoxView::setupContainer() {
	getContainer()->set(Children(), 0, 0);
}

void GtkBoxView::setChildren(Property * property, Any value) {
	property->setInert(value);
	const Point & capacity= *findProperty("capacity")->getInert()->getPoint();
	getContainer()->set(makeChildren(value), capacity.x, capacity.y);
}

void GtkBoxView::setCapacityProperty(Property * property, Any value) {
	Point capacity= *value->getPoint();
	setCapacity(&capacity.x, &capacity.y);
	property->setInert(CAny::createPoint(capacity));
}

void GtkBoxView::setCapacity(int * capS, int * capE) {
	Children children= getChildren();
	int cs= 0, ce= 0;
	for(int i=0; i<SIGNED(children.size()); ++i)
		if(children[i]->place.b==0)
			cs= std::max(cs, children[i]->place.a + 1);
		else
			ce= std::max(ce, children[i]->place.a + 1);
	if(*capS < cs)
		*capS= cs;
	if(*capE < ce)
		*capE= ce;
	getContainer()->set(children, *capS, *capE);
}

void GtkBoxView::onAsContainerSetup() {
	if(isAsContainer()) {
		findProperty("capacity")->clearFlag(sfReadOnly);
	} else {
		findProperty("capacity")->
			setInert(CAny::createPoint(Point(0, 0))).
			setFlag(sfReadOnly);
	}
	touch("capacity");
}

// ButtonBoxChildView ----------------------------------------------------------

ButtonBoxChildView::ButtonBoxChildView() {
	addProperty("secondary", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &ButtonBoxChildView::getSecondary)).
		setValueSet(sigc::mem_fun(*this, &ButtonBoxChildView::setSecondary));
}

Any ButtonBoxChildView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<ButtonBoxChild>());
}

Any ButtonBoxChildView::getSecondary(Property * property) {
	return CAny::createBool(getObject<ButtonBoxChild>()->secondary);
}

void ButtonBoxChildView::setSecondary(Property * property, Any value) {
	getObject<ButtonBoxChild>()->secondary= value->getBool();
}

// GtkButtonBoxView ------------------------------------------------------------

GtkButtonBoxView::GtkButtonBoxView() {
	setChildType("CrowButtonBoxChild");
	findProperty("homogeneous")->setFlag(sfReadOnly);
	addProperty("layout-style", prScalar, "GtkButtonBoxStyle", CAny::createEnum("GtkButtonBoxStyle", Gtk::BUTTONBOX_DEFAULT_STYLE));
}

void GtkButtonBoxView::initInstance() {
	GtkBoxView::initInstance();
	getObject<Gtk::ButtonBox>()->set_homogeneous(true);
	getObject<Gtk::ButtonBox>()->set_spacing(0);
}

ButtonBoxContainer * GtkButtonBoxView::createContainer() {
	return new ButtonBoxContainer();
}

// PanedChildView --------------------------------------------------------------

PanedChildView::PanedChildView() {
	addIndexProperty();
	addProperty("resize", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &PanedChildView::getResize)).
		setValueSet(sigc::mem_fun(*this, &PanedChildView::setResize));
	addProperty("shrink", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &PanedChildView::getShrink)).
		setValueSet(sigc::mem_fun(*this, &PanedChildView::setShrink));
}

Any PanedChildView::getResize(Property * property) {
	return CAny::createBool(getObject()->resize);
}

void PanedChildView::setResize(Property * property, Any value) {
	getObject()->resize= value->getBool();
}

Any PanedChildView::getShrink(Property * property) {
	return CAny::createBool(getObject()->shrink);
}

void PanedChildView::setShrink(Property * property, Any value) {
	getObject()->shrink= value->getBool();
}

// GtkPanedView ----------------------------------------------------------------

GtkPanedView::GtkPanedView() {
	setChildType("CrowPanedChild");
	addCanFocusProperty(true);
	addOrientationProperty();
	addProperty("position-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkPanedView::setPositionSet));
	addProperty("position", prScalar, "int").
		setFlag(sfReadOnly | sfNoUndo);
}

PNode GtkPanedView::createChild(const Place & place, PNode widget) {
	PNode child= GtkContainerView::createChild(place, widget);
	getModel()->createScalar(child, "index", CAny::createInt(place.a));
	return child;
}

Slider GtkPanedView::getSlider(Manipulator manip) {
	Slider slider;
	Glib::RefPtr<Gtk::Paned> obj= getObject<Gtk::Paned>();
	if(
		(
			(manip==SliderHor && getObject<Gtk::HPaned>()) ||
			(manip==SliderVer && getObject<Gtk::VPaned>())
		) &&
			obj->property_position_set()
	) {
		slider.manip= manip;
		int D= obj->property_max_position() - obj->property_min_position();
		if(D > 0) {
			double V= obj->property_position() - obj->property_min_position();
			slider.value= Cap(V / D, 0.0, 1.0);
		}
	}
	return slider;
}

void GtkPanedView::setSlider(Manipulator manip, double value) {
	Glib::RefPtr<Gtk::Paned> obj= getObject<Gtk::Paned>();
	if(obj->property_position_set()) {
		int D= obj->property_max_position() - obj->property_min_position();
		if(D > 0)
			obj->property_position()= Round(value*D + obj->property_min_position());
	}
}

void GtkPanedView::initDesign() {
	if(isRoot())
		setupAsContainer(true);
}

void GtkPanedView::setupContainer() {
	getContainer()->set(Children(), placeholdersSet());
}

void GtkPanedView::setChildren(Property * property, Any value) {
	property->setInert(value);
	getContainer()->set(makeChildren(value), placeholdersSet());
}

void GtkPanedView::setPositionSet(Property * property, Any value) {
	bool set= value->getBool();
	setInteractive(set);
	if(set)
		findProperty("position")->clearFlag(sfReadOnly);
	else
		findProperty("position")->setFlag(sfReadOnly);
	touch("position");
	getObject<Gtk::Paned>()->property_position_set()= set;
}

// PagesViewBase ---------------------------------------------------------------

PagesViewBase::PagesViewBase() {
	addInertProperty("capacity", prScalar, "int", CAny::createInt(0)).
		setFlag(sfReadOnly).
		setValueSet(sigc::mem_fun(*this, &PagesViewBase::setCapacityProperty));
	moveBefore("capacity", "children");
	addProperty("page", prScalar, "int", CAny::createInt(-1)).
		setFlag(sfNoUndo);
}

Slider PagesViewBase::getSlider(Manipulator manip) {
	Slider slider;
	if(appropManip(manip)) {
		slider.manip= manip;
		slider.steps= std::max(getPagesContainer()->getPageCount(), 1);
		if(slider.steps > 1)
			slider.value= double(getPagesContainer()->getCurrentPage()) / (slider.steps - 1);
		else
			slider.value= 0;
	}
	return slider;
}

void PagesViewBase::setSlider(Manipulator manip, double value) {
	int n= getPagesContainer()->getPageCount();
	if(n > 0) {
		int p= Round(value * (n-1));
		if(0 <= p && p <= n-1)
			getPagesContainer()->setCurrentPage(p);
	}
}

PNode PagesViewBase::createChild(const Place & place, PNode widget) {
	Model * model= getModel();
	PNode child= GtkContainerView::createChild(place, widget);
	model->createScalar(child, "index", CAny::createInt(place.a));
	return child;
}

void PagesViewBase::initDesign() {
	if(isRoot()) {
		setupAsContainer(true);
		getPagesContainer()->set(Children(), 3);
		findProperty("capacity")->setInert(CAny::createInt(3));
	}
}

void PagesViewBase::setupContainer() {
	getPagesContainer()->set(Children(), 0);
}

void PagesViewBase::setChildren(Property * property, Any value) {
	property->setInert(value);
	int capacity= findProperty("capacity")->getInert()->getInt();
	getPagesContainer()->set(makeChildren(value), capacity);
}

void PagesViewBase::setCapacityProperty(Property * property, Any value) {
	int capacity= value->getInt();
	setCapacity(&capacity);
	property->setInert(CAny::createInt(capacity));
}

void PagesViewBase::setCapacity(int * capacity) {
	Children children= getChildren();
	int cap= 0;
	for(int i=0; i<SIGNED(children.size()); ++i)
		cap= std::max(cap, children[i]->place.a + 1);
	if(*capacity < cap)
		*capacity= cap;
	getPagesContainer()->set(children, *capacity);
}

void PagesViewBase::onAsContainerSetup() {
	if(isAsContainer()) {
		findProperty("capacity")->clearFlag(sfReadOnly);
	} else {
		findProperty("capacity")->
			setInert(CAny::createInt(0)).
			setFlag(sfReadOnly);
	}
	touch("capacity");
}

// NotebookChildView -----------------------------------------------------------

NotebookChildView::NotebookChildView() {
	addIndexProperty();
	addProperty("tab-label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &NotebookChildView::getTabLabel)).
		setValueSet(sigc::mem_fun(*this, &NotebookChildView::setTabLabel));
	addProperty("tab-label-widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &NotebookChildView::getTabLabelWidget)).
		setValueSet(sigc::mem_fun(*this, &NotebookChildView::setTabLabelWidget));
	addProperty("menu-label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &NotebookChildView::getMenuLabel)).
		setValueSet(sigc::mem_fun(*this, &NotebookChildView::setMenuLabel));
	addProperty("menu-label-widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &NotebookChildView::getMenuLabelWidget)).
		setValueSet(sigc::mem_fun(*this, &NotebookChildView::setMenuLabelWidget));
	addProperty("pack-options", prScalar, "CrowPackOptions", CAny::createEnum("CrowPackOptions", Gtk::PACK_SHRINK)).
		setValueGet(sigc::mem_fun(*this, &NotebookChildView::getPackOptions)).
		setValueSet(sigc::mem_fun(*this, &NotebookChildView::setPackOptions));
	addInertProperty("detachable", prScalar, "bool", CAny::createBool(false));
	addInertProperty("reorderable", prScalar, "bool", CAny::createBool(false));
}

Any NotebookChildView::getTabLabel(Property * property) {
	return CAny::createString(getObject()->tabLabel);
}

void NotebookChildView::setTabLabel(Property * property, Any value) {
	getObject()->tabLabel= *value->getString();
}

Any NotebookChildView::getTabLabelWidget(Property * property) {
	return CAny::createObject(getObject()->tabLabelWidget);
}

void NotebookChildView::setTabLabelWidget(Property * property, Any value) {
	getObject()->tabLabelWidget= PGtkWidget::cast_dynamic(value->getObject());
}

Any NotebookChildView::getMenuLabel(Property * property) {
	return CAny::createString(getObject()->menuLabel);
}

void NotebookChildView::setMenuLabel(Property * property, Any value) {
	getObject()->menuLabel= *value->getString();
}

Any NotebookChildView::getMenuLabelWidget(Property * property) {
	return CAny::createObject(getObject()->menuLabelWidget);
}

void NotebookChildView::setMenuLabelWidget(Property * property, Any value) {
	getObject()->menuLabelWidget= PGtkWidget::cast_dynamic(value->getObject());
}

Any NotebookChildView::getPackOptions(Property * property) {
	return CAny::createEnum("CrowPackOptions", getObject()->packOptions);
}

void NotebookChildView::setPackOptions(Property * property, Any value) {
	getObject()->packOptions= checked_cast<Gtk::PackOptions>(value->getEnum());
}

// GtkNotebookView -------------------------------------------------------------

GtkNotebookView::GtkNotebookView() {
	setInteractive(true);
	setChildType("CrowNotebookChild");
	addCanFocusProperty(true);
	addProperty("enable-popup", prScalar, "bool", CAny::createBool(false));
	addProperty("homogeneous", prScalar, "bool", CAny::createBool(false));
	addProperty("scrollable", prScalar, "bool", CAny::createBool(false));
	addProperty("show-border", prScalar, "bool", CAny::createBool(true));
	addProperty("show-tabs", prScalar, "bool", CAny::createBool(true));
	addProperty("tab-hborder", prScalar, "unsigned", CAny::createUnsigned(2));
	addProperty("tab-vborder", prScalar, "unsigned", CAny::createUnsigned(2));
	addProperty("tab-pos", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_TOP));
	addInertProperty("group-id", prScalar, "int", CAny::createInt(-1));
}

bool GtkNotebookView::isTopChild(PChild child) {
	if(getObject()->get_current_page() == child->place.a) {
		CHECK(getObject()->get_current()->get_child() == GetPtr(child->widget));
		return true;
	} else
		return false;
}

bool GtkNotebookView::isActiveWidget(Gtk::Widget * widget) {
	if(getObject()->get_current()->get_child() == widget)
		return true;
	Children children= getChildren();
	bool tabs= getObject()->get_show_tabs();
	for(int i=0; i<SIGNED(children.size()); ++i) {
		PNotebookChild child= PNotebookChild::cast_dynamic(children[i]);
		if(widget==GetPtr(child->menuLabelWidget))
			return false;
		if(tabs && widget==GetPtr(child->tabLabelWidget))
			return true;
	}
	return false;
}

bool GtkNotebookView::appropManip(Manipulator manip) {
	Gtk::PositionType pos= getObject()->get_tab_pos();
	return
		(manip==SliderHor && (pos==Gtk::POS_TOP || pos==Gtk::POS_BOTTOM)) ||
		(manip==SliderVer && (pos==Gtk::POS_LEFT || pos==Gtk::POS_RIGHT))
	;
}

// AssistantChildView ----------------------------------------------------------

AssistantChildView::AssistantChildView() {
	addIndexProperty();
	addProperty("page-type", prScalar, "GtkAssistantPageType", CAny::createEnum("GtkAssistantPageType", Gtk::ASSISTANT_PAGE_CONTENT)).
		setValueGet(sigc::mem_fun(*this, &AssistantChildView::getPageType)).
		setValueSet(sigc::mem_fun(*this, &AssistantChildView::setPageType));
	addProperty("complete", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &AssistantChildView::getComplete)).
		setValueSet(sigc::mem_fun(*this, &AssistantChildView::setComplete));
	addProperty("title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &AssistantChildView::getTitle)).
		setValueSet(sigc::mem_fun(*this, &AssistantChildView::setTitle));
	addProperty("header-image", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string").
		setValueGet(sigc::mem_fun(*this, &AssistantChildView::getHeaderImage)).
		setValueSet(sigc::mem_fun(*this, &AssistantChildView::setHeaderImage));
	addProperty("sidebar-image", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string").
		setValueGet(sigc::mem_fun(*this, &AssistantChildView::getSidebarImage)).
		setValueSet(sigc::mem_fun(*this, &AssistantChildView::setSidebarImage));
}

Any AssistantChildView::getPageType(Property * property) {
	return CAny::createEnum("GtkAssistantPageType", getObject()->pageType);
}

void AssistantChildView::setPageType(Property * property, Any value) {
	getObject()->pageType= checked_cast<Gtk::AssistantPageType>(value->getEnum());
}

Any AssistantChildView::getComplete(Property * property) {
	return CAny::createBool(getObject()->complete);
}

void AssistantChildView::setComplete(Property * property, Any value) {
	getObject()->complete= value->getBool();
}

Any AssistantChildView::getTitle(Property * property) {
	return CAny::createString(getObject()->title);
}

void AssistantChildView::setTitle(Property * property, Any value) {
	getObject()->title= *value->getString();
}

Any AssistantChildView::getHeaderImage(Property * property) {
	return CAny::createString(getObject()->headerImage);
}

void AssistantChildView::setHeaderImage(Property * property, Any value) {
	getObject()->headerImage= *value->getString();
}

Any AssistantChildView::getSidebarImage(Property * property) {
	return CAny::createString(getObject()->sidebarImage);
}

void AssistantChildView::setSidebarImage(Property * property, Any value) {
	getObject()->sidebarImage= *value->getString();
}

// GtkAssistantViewBase --------------------------------------------------------

GtkAssistantViewBase::GtkAssistantViewBase() {
	setInteractive(true);
	setChildType("CrowAssistantChild");
	findProperty("page")->
		setValueGet(sigc::mem_fun(*this, &GtkAssistantViewBase::getPage)).
		setValueSet(sigc::mem_fun(*this, &GtkAssistantViewBase::setPage));
	findProperty("border-width")->setFlag(sfReadOnly);
	findProperty("focus-chain")->setFlag(sfNoEditing|sfHidden);
}

bool GtkAssistantViewBase::isTopChild(PChild child) {
	int page= getObject()->get_current_page();
	if(page == child->place.a) {
		CHECK(getObject()->get_nth_page(page) == GetPtr(child->widget));
		return true;
	} else
		return false;
}

bool GtkAssistantViewBase::isActiveWidget(Gtk::Widget * widget) {
	int page= getObject()->get_current_page();
	if(getObject()->get_nth_page(page) == widget)
		return true;
	else
		return false;
}

Any GtkAssistantViewBase::getPage(Property * property) {
	return CAny::createInt(getObject()->get_current_page());
}

void GtkAssistantViewBase::setPage(Property * property, Any value) {
	getObject()->set_current_page(value->getInt());
}

bool GtkAssistantViewBase::appropManip(Manipulator manip) {
	return manip==SliderHor;
}

// TableChildView --------------------------------------------------------------

TableChildView::TableChildView() {
	addProperty("cell", prScalar, "CrowPoint").
		setFlag(sfNoEditing | sfChildDef).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getCell)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setCell));
	addProperty("cell-span", prScalar, "CrowPoint", CAny::createPoint(Point(1, 1))).
		setFlag(sfChildDef).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getCellSpan)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setCellSpan));
	addProperty("padding-x", prScalar, "unsigned", CAny::createUnsigned(0)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getPaddingX)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setPaddingX));
	addProperty("padding-y", prScalar, "unsigned", CAny::createUnsigned(0)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getPaddingY)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setPaddingY));
	addProperty("x-expand", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getXExpand)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setXExpand));
	addProperty("x-fill", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getXFill)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setXFill));
	addProperty("x-shrink", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getXShrink)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setXShrink));
	addProperty("y-expand", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getYExpand)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setYExpand));
	addProperty("y-fill", prScalar, "bool", CAny::createBool(true)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getYFill)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setYFill));
	addProperty("y-shrink", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &TableChildView::getYShrink)).
		setValueSet(sigc::mem_fun(*this, &TableChildView::setYShrink));
}

Any TableChildView::getCell(Property * property) {
	return CAny::createPoint(Point(getObject()->place.a, getObject()->place.b));
}

void TableChildView::setCell(Property * property, Any value) {
	getObject()->place.a= value->getPoint()->x;
	getObject()->place.b= value->getPoint()->y;
}

Any TableChildView::getCellSpan(Property * property) {
	return CAny::createPoint(Point(getObject()->place.c, getObject()->place.d));
}

void TableChildView::setCellSpan(Property * property, Any value) {
	getObject()->place.c= value->getPoint()->x;
	getObject()->place.d= value->getPoint()->y;
}

Any TableChildView::getPaddingX(Property * property) {
	return CAny::createUnsigned(getObject()->paddingX);
}

void TableChildView::setPaddingX(Property * property, Any value) {
	getObject()->paddingX= value->getUnsigned();
}

Any TableChildView::getPaddingY(Property * property) {
	return CAny::createUnsigned(getObject()->paddingY);
}

void TableChildView::setPaddingY(Property * property, Any value) {
	getObject()->paddingY= value->getUnsigned();
}

Any TableChildView::getXExpand(Property * property) {
	return CAny::createBool(getObject()->xExpand);
}

void TableChildView::setXExpand(Property * property, Any value) {
	getObject()->xExpand= value->getBool();
}

Any TableChildView::getXFill(Property * property) {
	return CAny::createBool(getObject()->xFill);
}

void TableChildView::setXFill(Property * property, Any value) {
	getObject()->xFill= value->getBool();
}

Any TableChildView::getXShrink(Property * property) {
	return CAny::createBool(getObject()->xShrink);
}

void TableChildView::setXShrink(Property * property, Any value) {
	getObject()->xShrink= value->getBool();
}

Any TableChildView::getYExpand(Property * property) {
	return CAny::createBool(getObject()->yExpand);
}

void TableChildView::setYExpand(Property * property, Any value) {
	getObject()->yExpand= value->getBool();
}

Any TableChildView::getYFill(Property * property) {
	return CAny::createBool(getObject()->yFill);
}

void TableChildView::setYFill(Property * property, Any value) {
	getObject()->yFill= value->getBool();
}

Any TableChildView::getYShrink(Property * property) {
	return CAny::createBool(getObject()->yShrink);
}

void TableChildView::setYShrink(Property * property, Any value) {
	getObject()->yShrink= value->getBool();
}

// GtkTableView ----------------------------------------------------------------

GtkTableView::GtkTableView() {
	setChildType("CrowTableChild");
	addInertProperty("capacity", prScalar, "CrowPoint", CAny::createPoint(Point(1, 1))).
		setFlag(sfReadOnly).
		setValueSet(sigc::mem_fun(*this, &GtkTableView::setCapacityProperty));
	moveBefore("capacity", "children");
	addProperty("column-spacing", prScalar, "unsigned", CAny::createUnsigned(0));
	addProperty("homogeneous", prScalar, "bool", CAny::createBool(false));
	addProperty("row-spacing", prScalar, "unsigned", CAny::createUnsigned(0));
}

PNode GtkTableView::createChild(const Place & place, PNode widget) {
	Model * model= getModel();
	PNode child= GtkContainerView::createChild(place, widget);
	model->createScalar(child, "cell", CAny::createPoint(Point(place.a, place.b)));
	model->createScalar(child, "cell-span", CAny::createPoint(Point(place.c, place.d)));
	return child;
}

Any GtkTableView::createInstance() {
	Gtk::Table * table= new Gtk::Table(1, 1);
	return CAny::createObject(getType(), MakeRefPtr(table));
}

void GtkTableView::initDesign() {
	if(isRoot()) {
		setupAsContainer(true);
		getContainer()->set(Children(), 3, 3, true);
		findProperty("capacity")->setInert(CAny::createPoint(Point(3, 3)));
	}
}

void GtkTableView::setupContainer() {
	getContainer()->set(Children(), 1, 1, placeholdersSet());
}

void GtkTableView::setChildren(Property * property, Any value) {
	property->setInert(value);
	const Point & capacity= *findProperty("capacity")->getInert()->getPoint();
	getContainer()->set(makeChildren(value), capacity.x, capacity.y, placeholdersSet());
}

void GtkTableView::setCapacityProperty(Property * property, Any value) {
	Point capacity= *value->getPoint();
	setCapacity(&capacity.x, &capacity.y);
	property->setInert(CAny::createPoint(capacity));
}

void GtkTableView::setCapacity(int * capX, int * capY) {
	Children children= getChildren();
	int cx= 1, cy= 1;
	for(int i=0; i<SIGNED(children.size()); ++i) {
		const Place & place= children[i]->place;
		cx= std::max(cx, place.a + place.c);
		cy= std::max(cy, place.b + place.d);
	}
	if(*capX < cx)
		*capX= cx;
	if(*capY < cy)
		*capY= cy;
	getContainer()->set(children, *capX, *capY, placeholdersSet());
}

void GtkTableView::onAsContainerSetup() {
	if(isAsContainer()) {
		findProperty("capacity")->clearFlag(sfReadOnly);
	} else {
		findProperty("capacity")->
			setInert(CAny::createPoint(Point(1, 1))).
			setFlag(sfReadOnly);
	}
	touch("capacity");
}

// XYChildView -----------------------------------------------------------------

XYChildView::XYChildView() {
	addProperty("xy", prScalar, "CrowPoint").
		setFlag(sfChildDef).
		setValueGet(sigc::mem_fun(*this, &XYChildView::getXY)).
		setValueSet(sigc::mem_fun(*this, &XYChildView::setXY));
}

Any XYChildView::getXY(Property * property) {
	return CAny::createPoint(Point(getObject()->place.a, getObject()->place.b));
}

void XYChildView::setXY(Property * property, Any value) {
	getObject()->place.a= value->getPoint()->x;
	getObject()->place.b= value->getPoint()->y;
}

// XYContainerView -------------------------------------------------------------

XYContainerView::XYContainerView() {
	setChildType("CrowXYChild");
}

bool XYContainerView::findFreePlace(const Point & p, FreePlace * fplace) {
	if(!isAsContainer())
		return false;
	Point shift= dynamic_cast<XYContainer *>(getContainer())->getShift();
	int bw= getObject()->get_border_width();
	fplace->place.a= p.x - shift.x;
	fplace->place.b= p.y - shift.y;
	int width= getObject()->get_allocation().get_width();
	int height= getObject()->get_allocation().get_height();
	fplace->area= Rectangle(0, 0, width, height);
	fplace->area.grow(bw, bw, -2*bw, -2*bw);
	return true;
}

FreePlaces XYContainerView::findFreePlaces(const Point & p, const Nodes & nodes, const Point & p0) {

	if(!isAsContainer())
		return FreePlaces();

	Model * model= getModel();

	Nodes::const_iterator it= nodes.begin();
	PNode parent= model->findParent(*it);
	for(++it; it!=nodes.end(); ++it)
		if(parent != model->findParent(*it)) {
			parent= PNode();
			break;
		}

	FreePlaces fplaces;
	Point shift= dynamic_cast<XYContainer *>(getContainer())->getShift();

	if(parent && parent==getModelNode()) {

		fplaces.resize(nodes.size());
		int i= 0;
		for(Nodes::const_iterator it= nodes.begin(); it!=nodes.end(); ++it, ++i) {
			Rectangle R= getWidgetRect(getWidget(*it));
			R.x += p.x - p0.x;
			R.y += p.y - p0.y;
			fplaces[i].place.a= R.x - shift.x;
			fplaces[i].place.b= R.y - shift.y;
			fplaces[i].area= R;
		}

	} else {

		int si= 10, sy= 0;
		fplaces.resize(nodes.size());
		int i= 0;
		for(Nodes::const_iterator it= nodes.begin(); it!=nodes.end(); ++it, ++i) {
			int x= p.x;
			int y= p.y + sy;
			fplaces[i].place.a= x - shift.x;
			fplaces[i].place.b= y - shift.y;
			if(getController()->getView(*it)) {
				fplaces[i].area= Rectangle(Point(x, y), getWidgetPreferredSize(*it));
				sy += fplaces[i].area.h + si;
			} else
				sy += 2*si;
		}

	}

	return fplaces;
}

PNode XYContainerView::createChild(const Place & place, PNode widget) {
	Model * model= getModel();
	PNode child= GtkContainerView::createChild(place, widget);
	model->createScalar(child, "xy", CAny::createPoint(Point(place.a, place.b)));
	return child;
}

void XYContainerView::initDesign() {
	if(isRoot())
		setupAsContainer(true);
}

Point XYContainerView::getWidgetPreferredSize(PNode node) {
	Gtk::Requisition requisition= getWidget(node)->size_request();
	return Point(requisition.width, requisition.height);
}

PGdkWindow XYContainerView::getExposeWindow() {
	return getObject()->get_window();
}

bool XYContainerView::onExposeEvent(GdkEventExpose * event) {
	DrawGrid(getExposeWindow(), getObject()->get_style());
	return false;
}

void XYContainerView::onAsContainerSetup() {
	if(isAsContainer()) {
		expcon= getObject()->signal_expose_event().connect(sigc::mem_fun(*this, &XYContainerView::onExposeEvent), false);
	} else {
		expcon.disconnect();
	}
	getObject()->queue_draw();
}

// GtkAdjustmentView -----------------------------------------------------------

GtkAdjustmentView::GtkAdjustmentView() {
	addSignalsProperty();
	addProperty("lower", prScalar, "double", CAny::createDouble(0)).
		setValueSet(sigc::mem_fun(*this, &GtkAdjustmentView::setRangeProperty));
	addProperty("upper", prScalar, "double", CAny::createDouble(0)).
		setValueSet(sigc::mem_fun(*this, &GtkAdjustmentView::setRangeProperty));
	addProperty("page-size", prScalar, "double", CAny::createDouble(0)).
		setValueSet(sigc::mem_fun(*this, &GtkAdjustmentView::setRangeProperty));
	addProperty("page-increment", prScalar, "double", CAny::createDouble(0));
	addProperty("step-increment", prScalar, "double", CAny::createDouble(0));
	addProperty("value", prScalar, "double", CAny::createDouble(0));
}

void GtkAdjustmentView::setRangeProperty(Property * property, Any value) {
	setPropertyValue(property, value);
	double aval= getObject<Gtk::Adjustment>()->get_value();
	getObject<Gtk::Adjustment>()->set_value(aval);
}

void GtkAdjustmentView::configureForScrolling() {
	findProperty("lower")->setFlag(sfReadOnly | sfNoUndo);
	findProperty("page-increment")->setFlag(sfReadOnly | sfNoUndo);
	findProperty("page-size")->setFlag(sfReadOnly | sfNoUndo);
	findProperty("step-increment")->setFlag(sfReadOnly | sfNoUndo);
	findProperty("upper")->setFlag(sfReadOnly | sfNoUndo);
	findProperty("value")->setFlag(sfNoSave | sfNoUndo);
}

}
