/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#include "headers.h"
#include "forward.h"
#include "views2.h"

#include "palette.h"
#include "widgets.h"
#include "controller.h"

namespace Crow {

void RegisterViews2() {

	Palette * palette= Palette::get();

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GdkGravity")).
		addValue("GDK_GRAVITY_NORTH_WEST", Gdk::GRAVITY_NORTH_WEST).
		addValue("GDK_GRAVITY_NORTH", Gdk::GRAVITY_NORTH).
		addValue("GDK_GRAVITY_NORTH_EAST", Gdk::GRAVITY_NORTH_EAST).
		addValue("GDK_GRAVITY_WEST", Gdk::GRAVITY_WEST).
		addValue("GDK_GRAVITY_CENTER", Gdk::GRAVITY_CENTER).
		addValue("GDK_GRAVITY_EAST", Gdk::GRAVITY_EAST).
		addValue("GDK_GRAVITY_SOUTH_WEST", Gdk::GRAVITY_SOUTH_WEST).
		addValue("GDK_GRAVITY_SOUTH", Gdk::GRAVITY_SOUTH).
		addValue("GDK_GRAVITY_SOUTH_EAST", Gdk::GRAVITY_SOUTH_EAST).
		addValue("GDK_GRAVITY_STATIC", Gdk::GRAVITY_STATIC);

	palette->takeEntry(new EnumEntry("GtkWindowType")).
		addValue("GTK_WINDOW_TOPLEVEL", Gtk::WINDOW_TOPLEVEL).
		addValue("GTK_WINDOW_POPUP", Gtk::WINDOW_POPUP);

	palette->takeEntry(new EnumEntry("GdkWindowTypeHint")).
		addValue("GDK_WINDOW_TYPE_HINT_NORMAL", Gdk::WINDOW_TYPE_HINT_NORMAL).
		addValue("GDK_WINDOW_TYPE_HINT_DIALOG", Gdk::WINDOW_TYPE_HINT_DIALOG).
		addValue("GDK_WINDOW_TYPE_HINT_MENU", Gdk::WINDOW_TYPE_HINT_MENU).
		addValue("GDK_WINDOW_TYPE_HINT_TOOLBAR", Gdk::WINDOW_TYPE_HINT_TOOLBAR).
		addValue("GDK_WINDOW_TYPE_HINT_SPLASHSCREEN", Gdk::WINDOW_TYPE_HINT_SPLASHSCREEN).
		addValue("GDK_WINDOW_TYPE_HINT_UTILITY", Gdk::WINDOW_TYPE_HINT_UTILITY).
		addValue("GDK_WINDOW_TYPE_HINT_DOCK", Gdk::WINDOW_TYPE_HINT_DOCK).
		addValue("GDK_WINDOW_TYPE_HINT_DESKTOP", Gdk::WINDOW_TYPE_HINT_DESKTOP);

	palette->takeEntry(new EnumEntry("GtkWindowPosition")).
		addValue("GTK_WIN_POS_NONE", Gtk::WIN_POS_NONE).
		addValue("GTK_WIN_POS_CENTER", Gtk::WIN_POS_CENTER).
		addValue("GTK_WIN_POS_MOUSE", Gtk::WIN_POS_MOUSE).
		addValue("GTK_WIN_POS_CENTER_ALWAYS", Gtk::WIN_POS_CENTER_ALWAYS).
		addValue("GTK_WIN_POS_CENTER_ON_PARENT", Gtk::WIN_POS_CENTER_ON_PARENT);

	palette->takeEntry(new EnumEntry("GtkOrientation")).
		addValue("GTK_ORIENTATION_HORIZONTAL", Gtk::ORIENTATION_HORIZONTAL).
		addValue("GTK_ORIENTATION_VERTICAL", Gtk::ORIENTATION_VERTICAL);

	palette->takeEntry(new EnumEntry("GtkToolbarStyle")).
		addValue("GTK_TOOLBAR_ICONS", Gtk::TOOLBAR_ICONS).
		addValue("GTK_TOOLBAR_TEXT", Gtk::TOOLBAR_TEXT).
		addValue("GTK_TOOLBAR_BOTH", Gtk::TOOLBAR_BOTH).
		addValue("GTK_TOOLBAR_BOTH_HORIZ", Gtk::TOOLBAR_BOTH_HORIZ);

	palette->takeEntry(new TWidgetEntry<GtkWindowView>("GtkWindow", GTK_TYPE_WINDOW, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkResponseType")).
		addValue("GTK_RESPONSE_NONE", Gtk::RESPONSE_NONE).
		addValue("GTK_RESPONSE_REJECT", Gtk::RESPONSE_REJECT).
		addValue("GTK_RESPONSE_ACCEPT", Gtk::RESPONSE_ACCEPT).
		addValue("GTK_RESPONSE_DELETE_EVENT", Gtk::RESPONSE_DELETE_EVENT).
		addValue("GTK_RESPONSE_OK", Gtk::RESPONSE_OK).
		addValue("GTK_RESPONSE_CANCEL", Gtk::RESPONSE_CANCEL).
		addValue("GTK_RESPONSE_CLOSE", Gtk::RESPONSE_CLOSE).
		addValue("GTK_RESPONSE_YES", Gtk::RESPONSE_YES).
		addValue("GTK_RESPONSE_NO", Gtk::RESPONSE_NO).
		addValue("GTK_RESPONSE_APPLY", Gtk::RESPONSE_APPLY).
		addValue("GTK_RESPONSE_HELP", Gtk::RESPONSE_HELP);

	palette->takeEntry(new EnumEntry("GtkMessageType")).
		addValue("GTK_MESSAGE_INFO", Gtk::MESSAGE_INFO).
		addValue("GTK_MESSAGE_WARNING", Gtk::MESSAGE_WARNING).
		addValue("GTK_MESSAGE_QUESTION", Gtk::MESSAGE_QUESTION).
		addValue("GTK_MESSAGE_ERROR", Gtk::MESSAGE_ERROR).
		addValue("GTK_MESSAGE_OTHER", Gtk::MESSAGE_OTHER);

	palette->takeEntry(new TObjectEntry<DialogButtonEntryView>("CrowDialogButtonEntry", G_TYPE_INVALID));

	palette->takeEntry(new TWidgetEntry<GtkDialogView>("GtkDialog", GTK_TYPE_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkAssistantView>("GtkAssistant", GTK_TYPE_ASSISTANT, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TObjectEntry<InfoBarButtonEntryView>("CrowInfoBarButtonEntry", G_TYPE_INVALID));

	palette->takeEntry(new TWidgetEntry<GtkInfoBarView>("GtkInfoBar", GTK_TYPE_INFO_BAR, TAB_BINS));

	palette->takeEntry(new TWidgetEntry<GtkColorSelectionDialogView>("GtkColorSelectionDialog", GTK_TYPE_COLOR_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new EnumEntry("GtkFileChooserAction")).
		addValue("GTK_FILE_CHOOSER_ACTION_OPEN", Gtk::FILE_CHOOSER_ACTION_OPEN).
		addValue("GTK_FILE_CHOOSER_ACTION_SAVE", Gtk::FILE_CHOOSER_ACTION_SAVE).
		addValue("GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER).
		addValue("GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER", Gtk::FILE_CHOOSER_ACTION_CREATE_FOLDER);

	palette->takeEntry(new TObjectEntry<GtkFileFilterView>("GtkFileFilter", GTK_TYPE_FILE_FILTER));

	palette->takeEntry(new TWidgetEntry<GtkFileChooserDialogView>("GtkFileChooserDialog", GTK_TYPE_FILE_CHOOSER_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	palette->takeEntry(new TWidgetEntry<GtkFontSelectionDialogView>("GtkFontSelectionDialog", GTK_TYPE_FONT_SELECTION_DIALOG, TAB_WINDOWS)).
		setFlags(efTopMaster);

	// -----------------------------------

	palette->takeEntry(new EnumEntry("GtkPackDirection")).
		addValue("GTK_PACK_DIRECTION_LTR", GTK_PACK_DIRECTION_LTR).
		addValue("GTK_PACK_DIRECTION_RTL", GTK_PACK_DIRECTION_RTL).
		addValue("GTK_PACK_DIRECTION_TTB", GTK_PACK_DIRECTION_TTB).
		addValue("GTK_PACK_DIRECTION_BTT", GTK_PACK_DIRECTION_BTT);

	palette->takeEntry(new AbstractGtkWidgetEntry("CrowUIElement"));

	palette->takeEntry(new TWidgetEntry<GtkMenuBarView>("GtkMenuBar", GTK_TYPE_MENU_BAR, TAB_UI)).
		setIsA("CrowUIElement");

	palette->takeEntry(new TWidgetEntry<GtkToolbarView>("GtkToolbar", GTK_TYPE_TOOLBAR, TAB_UI)).
		setIsA("CrowUIElement");

	palette->takeEntry(new TWidgetEntry<GtkStatusbarView>("GtkStatusbar", GTK_TYPE_STATUSBAR, TAB_UI));

	palette->takeEntry(new TObjectEntry<GtkActionGroupView>("GtkActionGroup", GTK_TYPE_ACTION_GROUP, -2)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<GtkActionView>("GtkAction", GTK_TYPE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkToggleActionView>("GtkToggleAction", GTK_TYPE_TOGGLE_ACTION));

	palette->takeEntry(new TObjectEntry<GtkRadioActionView>("GtkRadioAction", GTK_TYPE_RADIO_ACTION));

	palette->takeEntry(new TObjectEntry<GtkUIManagerView>("GtkUIManager", GTK_TYPE_UI_MANAGER, -1)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	palette->takeEntry(new TObjectEntry<UIDefinitionView>("CrowUIDefinition", G_TYPE_INVALID, -2)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|CrowUIDefinition");

	palette->takeEntry(new TObjectEntry<RadioGroupView>("CrowRadioGroup", G_TYPE_INVALID, -3)).
		setPaletteTab(TAB_UI).
		setCanvasEditor("canvas|GlibObject");

	// -----------------------------------

	palette->takeEntry(new EnumEntry("PangoEllipsizeMode")).
		addValue("PANGO_ELLIPSIZE_NONE", Pango::ELLIPSIZE_NONE).
		addValue("PANGO_ELLIPSIZE_START", Pango::ELLIPSIZE_START).
		addValue("PANGO_ELLIPSIZE_MIDDLE", Pango::ELLIPSIZE_MIDDLE).
		addValue("PANGO_ELLIPSIZE_END", Pango::ELLIPSIZE_END);

	palette->takeEntry(new EnumEntry("GtkJustification")).
		addValue("GTK_JUSTIFY_LEFT", Gtk::JUSTIFY_LEFT).
		addValue("GTK_JUSTIFY_RIGHT", Gtk::JUSTIFY_RIGHT).
		addValue("GTK_JUSTIFY_CENTER", Gtk::JUSTIFY_CENTER).
		addValue("GTK_JUSTIFY_FILL", Gtk::JUSTIFY_FILL);

	palette->takeEntry(new EnumEntry("PangoWrapMode")).
		addValue("PANGO_WRAP_WORD", Pango::WRAP_WORD).
		addValue("PANGO_WRAP_CHAR", Pango::WRAP_CHAR).
		addValue("PANGO_WRAP_WORD_CHAR", Pango::WRAP_WORD_CHAR);

	palette->takeEntry(new TWidgetEntry<GtkLabelView>("GtkLabel", GTK_TYPE_LABEL, TAB_STANDARD));

	palette->takeEntry(new EnumEntry("GtkReliefStyle")).
		addValue("GTK_RELIEF_NORMAL", Gtk::RELIEF_NORMAL).
		addValue("GTK_RELIEF_HALF", Gtk::RELIEF_HALF).
		addValue("GTK_RELIEF_NONE", Gtk::RELIEF_NONE);

	palette->takeEntry(new TWidgetEntry<GtkButtonView>("GtkButton", GTK_TYPE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkToggleButtonView>("GtkToggleButton", GTK_TYPE_TOGGLE_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkCheckButtonView>("GtkCheckButton", GTK_TYPE_CHECK_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkRadioButtonView>("GtkRadioButton", GTK_TYPE_RADIO_BUTTON, TAB_STANDARD));

	palette->takeEntry(new TWidgetEntry<GtkScaleButtonView>("GtkScaleButton", GTK_TYPE_SCALE_BUTTON, TAB_SPECIAL));

}

// GtkWindowViewBase -----------------------------------------------------------

GtkWindowViewBase::GtkWindowViewBase() {
	addEventProperties();
	findProperty("visible")->
		setInertDefault(CAny::createBool(false)).
		setFlag(sfReadOnly);
	addInertProperty("accept-focus", prScalar, "bool", CAny::createBool(true));
	addInertProperty("default-size", prScalar, "CrowPoint", CAny::createPoint(Point(-1, -1)));
	addInertProperty("focus-on-map", prScalar, "bool", CAny::createBool(true));
	addInertProperty("gravity", prScalar, "GdkGravity", CAny::createEnum("GdkGravity", Gdk::GRAVITY_NORTH_WEST));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("modal", prScalar, "bool", CAny::createBool(false));
	addInertProperty("resizable", prScalar, "bool", CAny::createBool(true));
	addInertProperty("role", prScalar, "string", CAny::createString(""));
	addInertProperty("skip-pager-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("skip-taskbar-hint", prScalar, "bool", CAny::createBool(false));
	addProperty("title", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("window-position", prScalar, "GtkWindowPosition", CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_NONE));
	addInertProperty("transient-for", prEntity, "GtkWindow").
		setFlag(sfLinkOnly | sfWeakLink);
	addInertProperty("accel-groups", prVector, "GtkUIManager").
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertObject)).
		setFlag(sfLinkOnly);
	addInertProperty("urgency-hint", prScalar, "bool", CAny::createBool(false));
	addInertProperty("deletable", prScalar, "bool", CAny::createBool(true));
	addInertProperty("opacity", prScalar, "double", CAny::createDouble(1.0));
	addInertProperty("type-hint", prScalar, "GdkWindowTypeHint", CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_NORMAL));
}

void GtkWindowViewBase::initDesignTitle() {
	getObject()->set_property("title", getDesignLabel());
}

// GtkWindowView ---------------------------------------------------------------

GtkWindowView::GtkWindowView() {
	addAppPaintableProperty();
	addInertProperty("decorated", prScalar, "bool", CAny::createBool(true));
	addInertProperty("type", prScalar, "GtkWindowType", CAny::createEnum("GtkWindowType", Gtk::WINDOW_TOPLEVEL));
	addInertProperty("default", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
}

Any GtkWindowView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr(new Gtk::Window()));
}

void GtkWindowView::initDesign() {
	GtkBinView::initDesign();
	initDesignTitle();
}

// GtkAssistantView ------------------------------------------------------------

GtkAssistantView::GtkAssistantView() {
	addInertProperty("buttons", prVector, "GtkWidget").
		setValueSet(sigc::mem_fun(*this, &GtkAssistantView::setButtons)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
}

void GtkAssistantView::initDesign() {
	GtkAssistantViewBase::initDesign();
	initDesignTitle();
}

void GtkAssistantView::clearButtons() {
	while(!buttons.empty()) {
		PGtkWidget obj= buttons.front();
		buttons.pop_front();
		getObject()->remove_action_widget(*GetPtr(obj));
	}
}

void GtkAssistantView::setButtons(Property * property, Any value) {
	property->setInert(value);
	clearButtons();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkWidget obj= vec[i]->getObject<Gtk::Widget>();
		if(obj) {
			getObject()->add_action_widget(*GetPtr(obj));
			buttons.push_back(obj);
		}
	}
}

bool GtkAssistantView::isActiveWidget(Gtk::Widget * widget) {
	if(!GtkAssistantViewBase::isActiveWidget(widget)) {
		for(Buttons::iterator it= buttons.begin(); it!=buttons.end(); ++it)
			if(GetPtr(*it) == widget)
				return true;
		return false;
	} else
		return true;
}

// DialogButtonEntryView -------------------------------------------------------

DialogButtonEntry::DialogButtonEntry() {
	SetData(this, "CrowTypeHint", "CrowDialogButtonEntry");
	response= Gtk::RESPONSE_NONE;
	secondary= false;
}

DialogButtonEntryView::DialogButtonEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setWidget));
	addProperty("response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getResponse)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setResponse));
	addProperty("secondary", prScalar, "bool", CAny::createBool(false)).
		setValueGet(sigc::mem_fun(*this, &DialogButtonEntryView::getSecondary)).
		setValueSet(sigc::mem_fun(*this, &DialogButtonEntryView::setSecondary));
}

Any DialogButtonEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void DialogButtonEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any DialogButtonEntryView::getResponse(Property * property) {
	return CAny::createEnum("GtkResponseType", getObject()->response);
}

void DialogButtonEntryView::setResponse(Property * property, Any value) {
	getObject()->response= static_cast<Gtk::ResponseType>(value->getEnum());
}

Any DialogButtonEntryView::getSecondary(Property * property) {
	return CAny::createBool(getObject()->secondary);
}

void DialogButtonEntryView::setSecondary(Property * property, Any value) {
	getObject()->secondary= value->getBool();
}

// GtkDialogView ---------------------------------------------------------------

GtkDialogView::GtkDialogView() {
	addProperty("has-separator", prScalar, "bool", CAny::createBool(true));
	findProperty("type-hint")->
		setInertDefault(CAny::createEnum("GdkWindowTypeHint", Gdk::WINDOW_TYPE_HINT_DIALOG));
	findProperty("window-position")->
		setInertDefault(CAny::createEnum("GtkWindowPosition", Gtk::WIN_POS_CENTER_ON_PARENT));
	findProperty("type")->setFlag(sfReadOnly | sfHidden);
	findProperty("app-paintable")->setFlag(sfReadOnly | sfHidden);
	findProperty("default")->setFlag(sfReadOnly | sfHidden);
	addInertProperty("buttons", prVector, "CrowDialogButtonEntry").
		setFlag(sfOverwrite).
		setValueSet(sigc::mem_fun(*this, &GtkDialogView::setButtons)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GtkDialogView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkDialogView::getButtonLabel));
	addInertProperty("default-response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE));
	addInertProperty("alternative-button-order", prVector, "GtkResponseType").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertScalar),
			CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)
		));
}

BinContainer * GtkDialogView::createContainer() {
	return new DialogContainer();
}

Any GtkDialogView::createInstance() {
	Gtk::Dialog * window= new Gtk::Dialog();
	Gtk::Alignment * bin= Gtk::manage(new Gtk::Alignment());
	window->get_vbox()->pack_start(*bin);
	bin->show();
	return CAny::createObject(getType(), MakeRefPtr(window));
}

void GtkDialogView::setButtons(Property * property, Any value) {
	property->setInert(value);
	clearButtons();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PDialogButtonEntry entry= vec[i]->getObject<DialogButtonEntry>();
		if(entry && entry->widget)
			addButton(entry);
	}
}

void GtkDialogView::clearButtons() {
	ContainerClear(getObject<Gtk::Dialog>()->get_action_area());
}

void GtkDialogView::addButton(PDialogButtonEntry entry) {
	getObject<Gtk::Dialog>()->get_action_area()->add(*GetPtr(entry->widget));
	if(entry->secondary)
		getObject<Gtk::Dialog>()->get_action_area()->set_child_secondary(*GetPtr(entry->widget));
}

std::string GtkDialogView::getButtonLabel(Property * property, PNode node) {
	PNode resp= getModel()->find(node, "response");
	if(resp) {
		std::string str= resp->getValue()->toString();
		return Glib::ustring(str.substr(13)).lowercase();
	}
	return *node->getName();
}

// InfoBarButtonEntryView ------------------------------------------------------

InfoBarButtonEntry::InfoBarButtonEntry() {
	SetData(this, "CrowTypeHint", "CrowInfoBarButtonEntry");
	response= Gtk::RESPONSE_NONE;
}

InfoBarButtonEntryView::InfoBarButtonEntryView() {
	addProperty("widget", prEntity, "GtkWidget").
		setValueGet(sigc::mem_fun(*this, &InfoBarButtonEntryView::getWidget)).
		setValueSet(sigc::mem_fun(*this, &InfoBarButtonEntryView::setWidget));
	addProperty("response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)).
		setValueGet(sigc::mem_fun(*this, &InfoBarButtonEntryView::getResponse)).
		setValueSet(sigc::mem_fun(*this, &InfoBarButtonEntryView::setResponse));
}

Any InfoBarButtonEntryView::getWidget(Property * property) {
	return CAny::createObject(getObject()->widget);
}

void InfoBarButtonEntryView::setWidget(Property * property, Any value) {
	getObject()->widget= PGtkWidget::cast_dynamic(value->getObject());
}

Any InfoBarButtonEntryView::getResponse(Property * property) {
	return CAny::createEnum("GtkResponseType", getObject()->response);
}

void InfoBarButtonEntryView::setResponse(Property * property, Any value) {
	getObject()->response= static_cast<Gtk::ResponseType>(value->getEnum());
}

// GtkInfoBarView --------------------------------------------------------------

GtkInfoBarView::GtkInfoBarView() {
	addProperty("message-type", prScalar, "GtkMessageType", CAny::createEnum("GtkMessageType", Gtk::MESSAGE_INFO));
	addInertProperty("buttons", prVector, "CrowInfoBarButtonEntry").
		setFlag(sfOverwrite).
		setValueSet(sigc::mem_fun(*this, &GtkInfoBarView::setButtons)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GtkInfoBarView::insertEntity)).
		setLabelGet(sigc::mem_fun(*this, &GtkInfoBarView::getButtonLabel));
	addInertProperty("default-response", prScalar, "GtkResponseType", CAny::createEnum("GtkResponseType", Gtk::RESPONSE_NONE)).
		setFlag(sfReadOnly);
}

BinContainer * GtkInfoBarView::createContainer() {
	return new InfoBarContainer();
}

Any GtkInfoBarView::createInstance() {
	Gtk::InfoBar * widget= new Gtk::InfoBar();
	Gtk::Alignment * bin= Gtk::manage(new Gtk::Alignment());
	dynamic_cast<Gtk::Container *>(widget->get_content_area())->add(*bin);
	bin->show();
	return CAny::createObject(getType(), MakeRefPtr(widget));
}

void GtkInfoBarView::setButtons(Property * property, Any value) {
	property->setInert(value);
	clearButtons();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PInfoBarButtonEntry entry= vec[i]->getObject<InfoBarButtonEntry>();
		if(entry && entry->widget)
			addButton(entry);
	}
}

void GtkInfoBarView::clearButtons() {
	ContainerClear(getActionArea());
}

void GtkInfoBarView::addButton(PInfoBarButtonEntry entry) {
	getActionArea()->add(*GetPtr(entry->widget));
}

std::string GtkInfoBarView::getButtonLabel(Property * property, PNode node) {
	PNode resp= getModel()->find(node, "response");
	if(resp) {
		std::string str= resp->getValue()->toString();
		return Glib::ustring(str.substr(13)).lowercase();
	}
	return *node->getName();
}

Gtk::Container * GtkInfoBarView::getActionArea() {
	Glib::RefPtr<Gtk::InfoBar> obj= GlibObjectView::getObject<Gtk::InfoBar>();
	return dynamic_cast<Gtk::Container *>(obj->get_action_area());
}

// GtkDialogViewFinal ----------------------------------------------------------

GtkDialogViewFinal::GtkDialogViewFinal() {
	findProperty("buttons")->setFlag(sfReadOnly | sfHidden);
	findProperty("alternative-button-order")->setFlag(sfReadOnly | sfHidden);
	findProperty("as-container")->setFlag(sfReadOnly | sfHidden);
	findProperty("focus-chain")->setFlag(sfHidden);
	findProperty("default-response")->setFlag(sfReadOnly | sfHidden);
	findProperty("accel-groups")->setFlag(sfReadOnly | sfHidden);
	findProperty("title")->setFlag(sfAutoDefault);
}

// GtkColorSelectionDialogView -------------------------------------------------

GtkColorSelectionDialogView::GtkColorSelectionDialogView() {
	addProperty("colorsel", prEntity, "GtkColorSelection").
		setValueGet(sigc::mem_fun(*this, &GtkColorSelectionDialogView::getColorsel)).
		setViewConfigure(sigc::mem_fun(*this, &GtkColorSelectionDialogView::configureColorsel)).
		setFlag(sfReadOnly);
	findProperty("resizable")->setInertDefault(CAny::createBool(false));
	findProperty("has-separator")->setFlag(sfReadOnly);
	findProperty("border-width")->setFlag(sfReadOnly);
}

Any GtkColorSelectionDialogView::createInstance() {
	Gtk::ColorSelectionDialog * window= new Gtk::ColorSelectionDialog();
	window->get_colorsel()->set_current_color(Gdk::Color());
	return CAny::createObject(getType(), MakeRefPtr(window));
}

void GtkColorSelectionDialogView::configureColorsel(Property * property, PEntityView view) {
	view->findProperty("border-width")->setFlag(sfReadOnly);
}

Any GtkColorSelectionDialogView::getColorsel(Property * property) {
	PGtkWidget widget= MakeRefPtr(getObject<Gtk::ColorSelectionDialog>()->get_colorsel());
	return CAny::createObject(widget);
}

// GtkFileFilterView -----------------------------------------------------------

GtkFileFilterView::GtkFileFilterView() {
	addSignalsProperty();
	addProperty("name", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueGet(sigc::mem_fun(*this, &GtkFileFilterView::getName)).
		setValueSet(sigc::mem_fun(*this, &GtkFileFilterView::setName));
	addInertProperty("add-pixbuf-formats", prScalar, "bool", CAny::createBool(false));
	addInertProperty("patterns", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("*.*")
		));
	addInertProperty("mime-types", prVector, "string").
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*this, &GlibObjectView::insertScalar),
			CAny::createString("")
		));
}

Any GtkFileFilterView::createInstance() {
	Gtk::FileFilter * filter= new Gtk::FileFilter();
	filter->add_pattern("*.*");
	return CAny::createObject(getType(), MakeRefPtr(filter));
}

Any GtkFileFilterView::getName(Property * property) {
	return CAny::createString(getObject<Gtk::FileFilter>()->get_name());
}

void GtkFileFilterView::setName(Property * property, Any value) {
	getObject<Gtk::FileFilter>()->set_name(*value->getString());
}

// GtkFileChooserViewBase ------------------------------------------------------

GtkFileChooserViewBase::GtkFileChooserViewBase() {
	addProperty("action", prScalar, "GtkFileChooserAction", CAny::createEnum("GtkFileChooserAction", Gtk::FILE_CHOOSER_ACTION_OPEN));
	addProperty("extra-widget", prEntity, "GtkWidget");
	addInertProperty("file-system-backend", prScalar, "string", CAny::createString(""));
	addProperty("local-only", prScalar, "bool", CAny::createBool(true));
	addProperty("preview-widget", prEntity, "GtkWidget");
	addProperty("preview-widget-active", prScalar, "bool", CAny::createBool(true));
	addProperty("select-multiple", prScalar, "bool", CAny::createBool(false));
	addProperty("show-hidden", prScalar, "bool", CAny::createBool(false));
	addProperty("use-preview-label", prScalar, "bool", CAny::createBool(true));
	addInertProperty("filter", prEntity, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilter));
	addInertProperty("filters", prVector, "GtkFileFilter").
		setValueSet(sigc::mem_fun(*this, &GtkFileChooserViewBase::setFilters)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
	addProperty("do-overwrite-confirmation", prScalar, "bool", CAny::createBool(false));
	addProperty("create-folders", prScalar, "bool", CAny::createBool(true));
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserViewBase::getObject() {
	return Glib::RefPtr<Gtk::FileChooser>::cast_dynamic(GlibObjectView::getObject());
}

void GtkFileChooserViewBase::initInstance() {
	GtkWidgetView::initInstance();
	initFileChooserInstance();
}

void GtkFileChooserViewBase::initFileChooserInstance() {
	getObject()->set_current_folder(Glib::get_home_dir());
}

void GtkFileChooserViewBase::setFilter(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::FileFilter> filter= value->getObject<Gtk::FileFilter>();
	if(filter) {
		clearVectorProperty("filters")->setFlag(sfReadOnly);
		clearFilters();
	} else
		findProperty("filters")->clearFlag(sfReadOnly);
	touch("filters");
	getObject()->property_filter()= GetPtr(filter);
}

void GtkFileChooserViewBase::setFilters(Property * property, Any value) {
	property->setInert(value);
	clearFilters();
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		Glib::RefPtr<Gtk::FileFilter> obj= vec[i]->getObject<Gtk::FileFilter>();
		if(obj)
			chooser->add_filter(*GetPtr(obj));
	}
}

void GtkFileChooserViewBase::clearFilters() {
	Glib::RefPtr<Gtk::FileChooser> chooser= getObject();
	for(;;) {
		GSList * list= gtk_file_chooser_list_filters(chooser->gobj());
		if(list) {
			GtkFileFilter * filter= GTK_FILE_FILTER(list->data);
			gtk_file_chooser_remove_filter(chooser->gobj(), filter);
		} else
			break;
	}
}

bool GtkFileChooserViewBase::isActiveWidget(Gtk::Widget * widget) {
	if(
		widget==getObject()->get_preview_widget() &&
		!getObject()->get_preview_widget_active()
	)
		return false;
	else
		return true;
}

void GtkFileChooserViewBase::stopEmissionHandler(GtkWidget * widget, GtkWidget * prev, gpointer data) {
	g_signal_stop_emission(widget, GPOINTER_TO_UINT(data), 0);
}

void GtkFileChooserViewBase::fixWidgetSignals(GtkWidget * widget) {
	gpointer hierarchy= GUINT_TO_POINTER(g_signal_lookup("hierarchy-changed", GTK_TYPE_WIDGET));
	gpointer screen= GUINT_TO_POINTER(g_signal_lookup("screen-changed", GTK_TYPE_WIDGET));
	g_signal_connect(widget, "hierarchy-changed", G_CALLBACK(stopEmissionHandler), hierarchy);
	g_signal_connect(widget, "screen-changed", G_CALLBACK(stopEmissionHandler), screen);
}

void GtkFileChooserViewBase::fixFileChooserDefault(GtkWidget * widget, gpointer data) {
	if(GTK_IS_FILE_CHOOSER(widget))
		fixWidgetSignals(widget);
}

void GtkFileChooserViewBase::fixFileChooserWidget(GtkWidget * widget, gpointer data) {
	if(GTK_IS_FILE_CHOOSER_WIDGET(widget))
		gtk_container_forall(GTK_CONTAINER(widget), fixFileChooserDefault, NULL);
}

// GtkFileChooserDialogView ----------------------------------------------------

GtkFileChooserDialogView::GtkFileChooserDialogView() {
	findProperty("buttons")->clearFlag(sfReadOnly | sfHidden);
	findProperty("alternative-button-order")->clearFlag(sfReadOnly | sfHidden);
	findProperty("default-response")->clearFlag(sfReadOnly | sfHidden);
	findProperty("has-separator")->setInertDefault(CAny::createBool(false));
	findProperty("border-width")->setFlag(sfReadOnly);
}

Glib::RefPtr<Gtk::FileChooser> GtkFileChooserDialogView::getObject() {
	return GlibObjectView::getObject<Gtk::FileChooserDialog>();
}

Any GtkFileChooserDialogView::createInstance() {
	Gtk::FileChooserDialog * window= new Gtk::FileChooserDialog("");
	GtkContainer * container= GTK_CONTAINER(GTK_DIALOG(window->gobj())->vbox);
	gtk_container_forall(container, fixFileChooserWidget, NULL);
	return CAny::createObject(getType(), MakeRefPtr(window));
}

void GtkFileChooserDialogView::initInstance() {
	GtkDialogViewFinal::initInstance();
	GtkFileChooserViewBase::initFileChooserInstance();
}

void GtkFileChooserDialogView::initDesign() {
	getObject()->set_property("title", getDesignLabel());
}

// GtkFontSelectionDialogView --------------------------------------------------

GtkFontSelectionDialogView::GtkFontSelectionDialogView() {
	addProperty("font-selection", prEntity, "GtkFontSelection").
		setValueGet(sigc::mem_fun(*this, &GtkFontSelectionDialogView::getFontSelection)).
		setViewConfigure(sigc::mem_fun(*this, &GtkFontSelectionDialogView::configureFontSelection)).
		setFlag(sfReadOnly);
	findProperty("has-separator")->setFlag(sfReadOnly);
	findProperty("border-width")->setFlag(sfReadOnly);
}

Any GtkFontSelectionDialogView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr(new Gtk::FontSelectionDialog()));
}

void GtkFontSelectionDialogView::configureFontSelection(Property * property, PEntityView view) {
	view->findProperty("border-width")->setFlag(sfReadOnly);
}

Any GtkFontSelectionDialogView::getFontSelection(Property * property) {
	#if GTK_CHECK_VERSION(2, 20, 0)
		PGtkWidget widget= MakeRefPtr(Glib::wrap(getObject<Gtk::FontSelectionDialog>()->gobj()->fontsel));
	#else
		PGtkWidget widget= MakeRefPtr(getObject<Gtk::FontSelectionDialog>()->get_font_selection());
	#endif
	return CAny::createObject(widget);
}

// GtkStatusbarView ------------------------------------------------------------

GtkStatusbarView::GtkStatusbarView() {
	addProperty("has-resize-grip", prScalar, "bool", CAny::createBool(true));
}

// GtkActionGroupView ----------------------------------------------------------

GtkActionGroupView::GtkActionGroupView() {
	addSignalsProperty();
	addInertProperty("name", prScalar, "string", CAny::createString(""));
	addInertProperty("actions", prVector, "GtkAction").
		setValueSet(sigc::mem_fun(*this, &GtkActionGroupView::setActions)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject)).
		setLabelGet(sigc::mem_fun(*this, &GtkActionGroupView::getActionLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
}

int GtkActionGroupView::getNextId() {
	static int id = 0;
	return ++id;
}

Any GtkActionGroupView::createInstance() {
	Glib::ustring name= "GtkActionGroup_" + ToString(getNextId());
	return CAny::createObject(getType(), Gtk::ActionGroup::create(name));
}

void GtkActionGroupView::doneInstance() {
	clearActions();
	GlibObjectView::doneInstance();
}

void GtkActionGroupView::setActions(Property * property, Any value) {
	property->setInert(value);
	clearActions();
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkAction obj= vec[i]->getObject<Gtk::Action>();
		if(obj) {
			Any accel= getController()->findView(CAny::createObject(obj))->findProperty("accelerator")->getValue();
			if(accel->getString()->empty())
				group->add(obj);
			else
				group->add(obj, Gtk::AccelKey(*accel->getString()));
		}
	}
}

void GtkActionGroupView::clearActions() {
	PGtkActionGroup group= getObject<Gtk::ActionGroup>();
	for(;;) {
		GList * list= gtk_action_group_list_actions(group->gobj());
		if(list) {
			GtkAction * action= GTK_ACTION(list->data);
			gtk_action_group_remove_action(group->gobj(), action);
		} else
			break;
	}
}

std::string GtkActionGroupView::getActionLabel(Property * property, PNode node) {
	PEntityView view= getController()->getView(node);
	if(view)
		return "'" + *view->findProperty("name")->getValue()->getString() + "'";
	else
		return *node->getName();
}

// GtkActionView ---------------------------------------------------------------

GtkActionView::GtkActionView() {
	addSignalsProperty();
	addInertProperty("name", prScalar, "string", CAny::createString(""));
	addProperty("hide-if-empty", prScalar, "bool", CAny::createBool(true));
	addProperty("is-important", prScalar, "bool", CAny::createBool(false));
	addInertProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setLabel));
	addProperty("sensitive", prScalar, "bool", CAny::createBool(true));
	addProperty("stock-id", prScalar, "GtkStockID", CAny::create("GtkStockID", Gtk::StockID())).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setStockId));
	addProperty("tooltip", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("visible", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-horizontal", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-overflown", prScalar, "bool", CAny::createBool(true));
	addProperty("visible-vertical", prScalar, "bool", CAny::createBool(true));
	addInertProperty("accelerator", prScalar, "string", CAny::createString(""));
	addProperty("icon-name", prScalar, "string", CAny::createString("")).
		setEditor("icon-name|string");
	addInertProperty("short-label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("widgets", prVector, "GtkActivatable").
		setFlag(sfLinkOnly | sfWeakLink).
		setValueSet(sigc::mem_fun(*this, &GtkActionView::setWidgets)).
		setElementInsert(sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertObject));
}

Any GtkActionView::createInstance() {
	return CAny::createObject(getType(), Gtk::Action::create(createName()));
}

void GtkActionView::doneInstance() {
	clearWidgets();
	GlibObjectView::doneInstance();
}

Glib::ustring GtkActionView::createName() {
	static int counter= 0;
	return "$tmp$" + ToString(++counter);
}

void GtkActionView::setLabel(Property * property, Any value) {
	property->setInert(value);
	ustring_cr str= *value->getString();
	PGtkAction action= getObject<Gtk::Action>();
	if(str.empty()) {
		g_object_set(action->gobj(), "label", NULL, NULL);
		Gtk::StockID stock= action->property_stock_id();
		action->property_stock_id()= stock;
	} else
		action->property_label()= str;
}

void GtkActionView::setStockId(Property * property, Any value) {
	PGtkAction action= getObject<Gtk::Action>();
	const Gtk::StockID & id= *value->get<Gtk::StockID>();
	action->property_stock_id()= id;
	if(id == Gtk::StockID())
		findProperty("icon-name")->clearFlag(sfReadOnly);
	else {
		findProperty("icon-name")->setFlag(sfReadOnly);
		action->set_property("icon-name", Glib::ustring());
	}
	touch("icon-name");
}

void GtkActionView::clearWidgets() {
	while(!widgets.empty()) {
		Gtk::Widget * widget= GetPtr(widgets.front());
		widget->set_property("related-action", PGtkAction());
		if(GetProperty<bool>(widget, "use-action-appearance"))
			ResetAppearance(widget);
		widget->set_sensitive(GetData(widget, "CrowActivatableSensitive") != "false");
		widgets.pop_front();
	}
}

void GtkActionView::setWidgets(Property * property, Any value) {
	property->setInert(value);
	clearWidgets();
	Glib::RefPtr<Gtk::SizeGroup> sizeGroup= getObject<Gtk::SizeGroup>();
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkWidget obj= vec[i]->getObject<Gtk::Widget>();
		if(obj) {
			obj->set_property("related-action", getObject());
			widgets.push_back(obj);
		}
	}
}

// GtkToggleActionView ---------------------------------------------------------

GtkToggleActionView::GtkToggleActionView() {
	addProperty("draw-as-radio", prScalar, "bool", CAny::createBool(false));
	addProperty("active", prScalar, "bool", CAny::createBool(false));
}

Any GtkToggleActionView::createInstance() {
	return CAny::createObject(getType(), Gtk::ToggleAction::create(createName()));
}

// GtkRadioActionView ----------------------------------------------------------

GtkRadioActionView::GtkRadioActionView() {
	addProperty("value", prScalar, "int", CAny::createInt(0));
	findProperty("draw-as-radio")->
		setDefault(CAny::createBool(true)).
		setFlag(sfReadOnly | sfHidden);
	addInertProperty("group", prEntity, "CrowRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioActionView::setActive));
}

Any GtkRadioActionView::createInstance() {
	Gtk::RadioAction::Group group;
	dummy= Gtk::RadioAction::create(group, "");
	Glib::RefPtr<Gtk::RadioAction> action= Gtk::RadioAction::create(group, createName());
	action->property_draw_as_radio()= true;
	return CAny::createObject(getType(), action);
}

void GtkRadioActionView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioAction>()->set_active(value->getBool());
}

// GtkUIManagerView ------------------------------------------------------------

GtkUIManagerView::GtkUIManagerView() {
	addSignalsProperty();
	addProperty("add-tearoffs", prScalar, "bool", CAny::createBool(false));
	addInertProperty("action-groups", prVector, "GtkActionGroup").
		setFlag(sfLinkOnly).
		setValueSet(sigc::mem_fun(*this, &GtkUIManagerView::setActionGroups)).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
	addInertProperty("ui-definitions", prVector, "CrowUIDefinition").
		setFlag(sfLinkOnly).
		setElementInsert(sigc::mem_fun(*this, &GlibObjectView::insertObject));
}

Gtk::UIManager::ui_merge_id GtkUIManagerView::getMergeId(int index) {
	CHECK(0 <= index && index < SIGNED(mergeIds.size()));
	return mergeIds[index];
}

Any GtkUIManagerView::createInstance() {
	PGtkUIManager manager= Gtk::UIManager::create();
	return CAny::createObject(getType(), manager);
}

void GtkUIManagerView::setActionGroups(Property * property, Any value) {
	property->setInert(value);
	PGtkUIManager manager= getObject<Gtk::UIManager>();
	for(;;) {
		GList * list= gtk_ui_manager_get_action_groups(manager->gobj());
		if(list) {
			GtkActionGroup * group= GTK_ACTION_GROUP(list->data);
			gtk_ui_manager_remove_action_group(manager->gobj(), group);
		} else
			break;
	}
	const AnyVector & vec= *value->getVector();
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		PGtkActionGroup obj= vec[i]->getObject<Gtk::ActionGroup>();
		if(obj)
			manager->insert_action_group(obj);
	}
}

// UIDefinitionView ------------------------------------------------------------

UIDefinition::UIDefinition() :
	Glib::ObjectBase(typeid(UIDefinition)),
	propertyUi(*this, "ui", "<ui>\n</ui>\n")
{
	SetData(this, "CrowTypeHint", "CrowUIDefinition");
}

UIDefinitionView::UIDefinitionView() {
	addProperty("ui", prScalar, "string", CAny::createString("<ui>\n</ui>\n")).
		setFlag(sfHidden | sfNoEditing | sfXml);
}

// UIElementView ---------------------------------------------------------------

UIElementView::UIElementView() {
	//setReportDestruction(false);
	addInertProperty("element", prScalar, "string", CAny::createString("")).
		setEditor("element|string");
	addInertProperty("manager", prEntity, "GtkUIManager").
		setFlag(sfWeakLink | sfLinkOnly);
}

Any UIElementView::createInstance() {
	return CAny::createObject(getType(), createStub());
}

// GtkMenuBarView --------------------------------------------------------------

GtkMenuBarView::GtkMenuBarView() {
	addEventProperties();
	addProperty("take-focus", prScalar, "bool", CAny::createBool(true));
	addProperty("child-pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
	addProperty("pack-direction", prScalar, "GtkPackDirection", CAny::createEnum("GtkPackDirection", GTK_PACK_DIRECTION_LTR));
}

bool GtkMenuBarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::MenuBar *>(widget);
}

PGtkWidget GtkMenuBarView::createStub() {
	Gtk::MenuBar * menuBar= new Gtk::MenuBar();
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("File"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Edit"));
	menuBar->items().push_back(Gtk::Menu_Helpers::MenuElem("Help"));
	return MakeRefPtr(menuBar);
}

// GtkToolbarView --------------------------------------------------------------

GtkToolbarView::GtkToolbarView() {
	addOrientationProperty();
	addProperty("show-arrow", prScalar, "bool", CAny::createBool(true));
	addInertProperty("toolbar-style-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setToolbarStyleSet));
	addInertGetProperty("toolbar-style", prScalar, "GtkToolbarStyle", CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS)).
		setFlag(sfOverwrite | sfReadOnly);
	addInertProperty("tooltips", prScalar, "bool", CAny::createBool(true));
	addInertProperty("icon-size-set", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkToolbarView::setIconSizeSet));
	addProperty("icon-size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_LARGE_TOOLBAR)).
		setFlag(sfReadOnly);
}

bool GtkToolbarView::validWidget(Gtk::Widget * widget) {
	return dynamic_cast<Gtk::Toolbar *>(widget);
}

PGtkWidget GtkToolbarView::createStub() {
	Gtk::Toolbar * toolbar= new Gtk::Toolbar();
	Gtk::ToolButton * b1= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::NEW));
	Gtk::ToolButton * b2= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::OPEN));
	Gtk::ToolButton * b3= Gtk::manage(new Gtk::ToolButton(Gtk::Stock::SAVE));
	b1->show();
	b2->show();
	b3->show();
	toolbar->append(*b1);
	toolbar->append(*b2);
	toolbar->append(*b3);
	return MakeRefPtr(toolbar);
}

void GtkToolbarView::setToolbarStyleSet(Property * property, Any value) {
	property->setInert(value);
	if(value->getBool()) {
		findProperty("toolbar-style")->clearFlag(sfReadOnly);
		getObject<Gtk::Toolbar>()->set_toolbar_style(Gtk::TOOLBAR_ICONS);
	} else {
		findProperty("toolbar-style")->
			setFlag(sfReadOnly).
			setInert(CAny::createEnum("GtkToolbarStyle", Gtk::TOOLBAR_ICONS));
		getObject<Gtk::Toolbar>()->unset_toolbar_style();
	}
	touch("toolbar-style");
}

void GtkToolbarView::setIconSizeSet(Property * property, Any value) {
	property->setInert(value);
	bool set= value->getBool();
	getObject()->set_property("icon-size", Gtk::ICON_SIZE_LARGE_TOOLBAR);
	getObject()->set_property("icon-size-set", set);
	if(set)
		findProperty("icon-size")->clearFlag(sfReadOnly);
	else
		findProperty("icon-size")->setFlag(sfReadOnly);
	touch("icon-size");
}

// RadioGroupView --------------------------------------------------------------

RadioGroup::RadioGroup() {
	SetData(this, "CrowTypeHint", "CrowRadioGroup");
}

// GtkMiscView -----------------------------------------------------------------

GtkMiscView::GtkMiscView() {
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("xpad", prScalar, "int", CAny::createInt(0));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("ypad", prScalar, "int", CAny::createInt(0));
}

// GtkLabelView ----------------------------------------------------------------

GtkLabelView::GtkLabelView() {
	addProperty("angle", prScalar, "double", CAny::createDouble(0.0));
	addProperty("ellipsize", prScalar, "PangoEllipsizeMode", CAny::createEnum("PangoEllipsizeMode", Pango::ELLIPSIZE_NONE));
	addProperty("justify", prScalar, "GtkJustification", CAny::createEnum("GtkJustification", Gtk::JUSTIFY_LEFT));
	addProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addProperty("max-width-chars", prScalar, "int", CAny::createInt(-1));
	addProperty("mnemonic-widget", prEntity, "GtkWidget").
		setFlag(sfLinkOnly | sfWeakLink);
	addProperty("selectable", prScalar, "bool", CAny::createBool(false));
	addProperty("single-line-mode", prScalar, "bool", CAny::createBool(false));
	addProperty("use-markup", prScalar, "bool", CAny::createBool(false));
	addProperty("use-underline", prScalar, "bool", CAny::createBool(false));
	addProperty("wrap-mode", prScalar, "PangoWrapMode", CAny::createEnum("PangoWrapMode", Pango::WRAP_WORD));
	addProperty("wrap", prScalar, "bool", CAny::createBool(false));
	addProperty("track-visited-links", prScalar, "bool", CAny::createBool(true));
}

void GtkLabelView::initDesign() {
	getObject()->set_label(getDesignLabel());
}

// GtkButtonViewBase -----------------------------------------------------------

GtkButtonViewBase::GtkButtonViewBase() {
	addReceivesDefaultProperty(true);
	addCanFocusProperty(true);
	addCanDefaultProperty(false);
	addProperty("focus-on-click", prScalar, "bool", CAny::createBool(true));
	addProperty("relief", prScalar, "GtkReliefStyle", CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NORMAL));
	addProperty("xalign", prScalar, "float", CAny::createFloat(0.5f));
	addProperty("yalign", prScalar, "float", CAny::createFloat(0.5f));
	findProperty("sensitive")->
		setValueSet(sigc::mem_fun(*this, &GtkButtonViewBase::setSensitive));
}

void GtkButtonViewBase::initInstance() {
	GtkWidgetView::initInstance();
	setUAA(false);
}

void GtkButtonViewBase::setSensitive(Property * property, Any value) {
	property->setInert(value);
	Gtk::Widget * widget= GetPtr(getObject());
	bool sensitive= value->getBool();
	if(!GetProperty<PGtkAction>(widget, "related-action"))
		widget->set_sensitive(sensitive);
	SetData(widget, "CrowActivatableSensitive", sensitive ? "true" : "false");
}

void GtkButtonViewBase::setUAA(bool uaa) {
	gtk_activatable_set_use_action_appearance(GTK_ACTIVATABLE(getObject()->gobj()), uaa);
}

bool GtkButtonViewBase::getUAA() {
	return gtk_activatable_get_use_action_appearance(GTK_ACTIVATABLE(getObject()->gobj()));
}

// GtkButtonView ---------------------------------------------------------------

GtkButtonView::GtkButtonView() {
	addInertProperty("use-action-appearance", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setUseActionAppearance));
	addInertProperty("stock-id", prScalar, "GtkStockID", CAny::create("GtkStockID", Gtk::StockID())).
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setStockId));
	addInertGetProperty("label", prScalar, "string", CAny::createString("")).
		setFlag(sfTranslatable);
	addInertProperty("image", prEntity, "GtkWidget").
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setImage));
	addInertProperty("use-underline", prScalar, "bool", CAny::createBool(false)).
		setValueSet(sigc::mem_fun(*this, &GtkButtonView::setUseUnderline));
	addProperty("image-position", prScalar, "GtkPositionType", CAny::createEnum("GtkPositionType", Gtk::POS_LEFT));
}

void GtkButtonView::initInstance() {
	GtkBinView::initInstance();
	setUAA(false);
}

void GtkButtonView::initDesign() {
	findProperty("label")->setValue(CAny::createString(getDesignLabel()));
}

void GtkButtonView::setupContainer() {
	if(isAsContainer()) {
		setUAA(false);
		resetButton();
		BinRemove(GetPtr(getObject()));
		GtkBinView::setupContainer();
	} else {
		CHECK(!getUAA());
		BinRemove(GetPtr(getObject()));
		resetButton();
	}
}

void GtkButtonView::onAsContainerSetup() {
	setState(isAsContainer() ? bsContainer : bsNormal);
}

void GtkButtonView::resetButton() {
	ResetAppearance(GetPtr(getObject()));
}

void GtkButtonView::setState(ButtonState bs) {

	switch(bs) {

		case bsNormal:
			findProperty("use-action-appearance")->clearFlag(sfReadOnly);
			findProperty("label")->clearFlag(sfReadOnly);
			findProperty("image")->clearFlag(sfReadOnly);
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->clearFlag(sfReadOnly);
			break;

		case bsActivatable:
			findProperty("use-action-appearance")->clearFlag(sfReadOnly);
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->setFlag(sfReadOnly).setInert(CAny::create("GtkStockID", Gtk::StockID()));
			findProperty("use-underline")->setFlag(sfReadOnly).setInert(CAny::createBool(false));
			break;

		case bsStock:
			findProperty("use-action-appearance")->setFlag(sfReadOnly).setInert(CAny::createBool(false));
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->clearFlag(sfReadOnly);
			findProperty("use-underline")->setFlag(sfReadOnly).setInert(CAny::createBool(false));
			break;

		case bsContainer:
			findProperty("use-action-appearance")->setFlag(sfReadOnly).setInert(CAny::createBool(false));
			findProperty("label")->setFlag(sfReadOnly).setInert(CAny::createString(""));
			findProperty("image")->setFlag(sfReadOnly).setInert(CAny::createObject());
			findProperty("stock-id")->setFlag(sfReadOnly).setInert(CAny::create("GtkStockID", Gtk::StockID()));
			findProperty("use-underline")->setFlag(sfReadOnly).setInert(CAny::createBool(false));
			break;

		default:
			CHECK(false);

	}

	touch("use-action-appearance");
	touch("label");
	touch("image");
	touch("stock-id");
	touch("use-underline");
}

void GtkButtonView::setUseActionAppearance(Property * property, Any value) {
	property->setInert(value);
	bool uaa= value->getBool();
	setState(!uaa ? bsNormal : bsActivatable);
	resetButton();
	setUAA(uaa);
}

void GtkButtonView::setStockId(Property * property, Any value) {
	CHECK(!getUAA());
	property->setInert(value);
	Glib::ustring str= value->get<Gtk::StockID>()->get_string();
	setState(str.empty() ? bsNormal : bsStock);
	resetButton();
	if(!str.empty()) {
		getObject()->set_label(str);
		getObject()->set_use_stock(true);
	}
}

void GtkButtonView::setImage(Property * property, Any value) {
	property->setInert(value);
	CHECK(!getUAA() && !getObject()->get_use_stock() && !isAsContainer());
	getObject()->property_image()= GetPtr(value->getObject<Gtk::Widget>());
}

void GtkButtonView::setUseUnderline(Property * property, Any value) {
	property->setInert(value);
	CHECK(!getUAA() && !getObject()->get_use_stock() && !isAsContainer());
	getObject()->set_use_underline(value->getBool());
}

bool GtkButtonView::isActiveWidget(Gtk::Widget * widget) {
	if(getObject()->property_image() == widget)
		return Gtk::Settings::get_default()->property_gtk_button_images();
	else
		return true;
}

// GtkToggleButtonView ---------------------------------------------------------

GtkToggleButtonView::GtkToggleButtonView() {
	addProperty("active", prScalar, "bool", CAny::createBool(false));
	addProperty("inconsistent", prScalar, "bool", CAny::createBool(false));
}

Any GtkToggleButtonView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<Gtk::ToggleButton>());
}

// GtkCheckButtonView ----------------------------------------------------------

GtkCheckButtonView::GtkCheckButtonView() {
	addReceivesDefaultProperty(false);
	addProperty("draw-indicator", prScalar, "bool", CAny::createBool(true));
}

Any GtkCheckButtonView::createInstance() {
	return CAny::createObject(getType(), MakeRefPtr<Gtk::CheckButton>());
}

// GtkRadioButtonView ----------------------------------------------------------

GtkRadioButtonView::GtkRadioButtonView() {
	addInertProperty("group", prEntity, "CrowRadioGroup").
		setFlag(sfWeakLink | sfLinkOnly);
	findProperty("active")->
		setValueSet(sigc::mem_fun(*this, &GtkRadioButtonView::setActive));
}

Any GtkRadioButtonView::createInstance() {
	Gtk::RadioButton::Group group;
	dummy= MakeRefPtr(new Gtk::RadioButton(group));
	Glib::RefPtr<Gtk::RadioButton> button= MakeRefPtr(new Gtk::RadioButton(group));
	return CAny::createObject(getType(), button);
}

void GtkRadioButtonView::setActive(Property * property, Any value) {
	dummy->set_active(!value->getBool());
	getObject<Gtk::RadioButton>()->set_active(value->getBool());
}

// GtkScaleButtonView ----------------------------------------------------------

GtkScaleButtonView::GtkScaleButtonView() {
	setInteractive(true);
	addOrientationProperty();
	findProperty("relief")->setDefault(CAny::createEnum("GtkReliefStyle", Gtk::RELIEF_NONE));
	findProperty("focus-on-click")->setDefault(CAny::createBool(false));
	addProperty("adjustment", prEntity, "GtkAdjustment").
		setFlag(sfReadOnly);
	addProperty("size", prScalar, "GtkIconSize", CAny::createEnum("GtkIconSize", Gtk::ICON_SIZE_SMALL_TOOLBAR));
	addInertProperty("icons", prVector, "string").
		setValueSet(sigc::mem_fun(*this, &GtkScaleButtonView::setIcons)).
		setElementInsert(sigc::bind<Any>(
			sigc::mem_fun(*dynamic_cast<GlibObjectView *>(this), &GlibObjectView::insertScalar),
			CAny::createString("")
		)).
		setElementEditor("icon-name|string");
}

Any GtkScaleButtonView::createInstance() {
	GtkWidget * widget= gtk_scale_button_new(GTK_ICON_SIZE_SMALL_TOOLBAR, 0, 0, 0, NULL);
	Gtk::ScaleButton * button= Glib::wrap(GTK_SCALE_BUTTON(widget));
	return CAny::createObject(getType(), MakeRefPtr(button));
}

void GtkScaleButtonView::setIcons(Property * property, Any value) {
	property->setInert(value);
	Glib::RefPtr<Gtk::ScaleButton> button= GlibObjectView::getObject<Gtk::ScaleButton>();
	const AnyVector & vec= *value->getVector();
	std::list<Glib::ustring> icons;
	for(int i=0; i<SIGNED(vec.size()); ++i) {
		ustring_cr str= *vec[i]->getString();
		icons.push_back(str);
	}
	button->set_icons(icons);
}

}
