/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_VIEWS2_H
#define CROW_VIEWS2_H

#include "views1.h"

namespace Crow {

class GtkWindowViewBase : public virtual GtkContainerView {

public:
	GtkWindowViewBase();

protected:
	void initDesignTitle();

};

class GtkWindowView : public GtkBinView, public GtkWindowViewBase {

public:
	using GlibObjectView::getObject;

	GtkWindowView();

protected:
	Any createInstance();
	void initDesign();

};

class GtkAssistantView : public GtkAssistantViewBase, public GtkWindowViewBase {

public:
	GtkAssistantView();

protected:
	void initDesign();

	void clearButtons();
	void setButtons(Property * property, Any value);

	bool isActiveWidget(Gtk::Widget * widget);

private:
	typedef std::list<PGtkWidget> Buttons;
	Buttons buttons;

};

class DialogButtonEntry : public Glib::Object {

public:
	DialogButtonEntry();

	PGtkWidget widget;
	Gtk::ResponseType response;
	bool secondary;
	
};

typedef Glib::RefPtr<DialogButtonEntry> PDialogButtonEntry;

class DialogButtonEntryView : public TCreateInstance<DialogButtonEntry> {

public:
	DialogButtonEntryView();

protected:
	Any getWidget(Property * property);
	void setWidget(Property * property, Any value);
	Any getResponse(Property * property);
	void setResponse(Property * property, Any value);
	Any getSecondary(Property * property);
	void setSecondary(Property * property, Any value);

};

class GtkDialogView : public GtkWindowView {

public:
	GtkDialogView();

protected:
	BinContainer * createContainer();
	Any createInstance();

	void setButtons(Property * property, Any value);
	void clearButtons();
	void addButton(PDialogButtonEntry entry);

	std::string getButtonLabel(Property * property, PNode node);

};

class InfoBarButtonEntry : public Glib::Object {

public:
	InfoBarButtonEntry();

	PGtkWidget widget;
	Gtk::ResponseType response;

};

typedef Glib::RefPtr<InfoBarButtonEntry> PInfoBarButtonEntry;

class InfoBarButtonEntryView : public TCreateInstance<InfoBarButtonEntry> {

public:
	InfoBarButtonEntryView();

protected:
	Any getWidget(Property * property);
	void setWidget(Property * property, Any value);
	Any getResponse(Property * property);
	void setResponse(Property * property, Any value);

};

class GtkInfoBarView : public GtkBinView {

public:
	GtkInfoBarView();

protected:
	BinContainer * createContainer();
	Any createInstance();

	void setButtons(Property * property, Any value);
	void clearButtons();
	void addButton(PInfoBarButtonEntry entry);

	std::string getButtonLabel(Property * property, PNode node);
	Gtk::Container * getActionArea();

};

class GtkDialogViewFinal : public GtkDialogView {

public:
	GtkDialogViewFinal();
	
protected:
	void initDesign() { }

};

class GtkColorSelectionDialogView : public GtkDialogViewFinal {

public:
	GtkColorSelectionDialogView();

protected:
	Any createInstance();
	void configureColorsel(Property * property, PEntityView view);
	Any getColorsel(Property * property);

};

class GtkFileFilterView : public GlibObjectView {

public:
	GtkFileFilterView();

protected:
	Any createInstance();
	Any getName(Property * property);
	void setName(Property * property, Any value);

};

class GtkFileChooserViewBase : public virtual GtkWidgetView {

public:
	GtkFileChooserViewBase();

	virtual Glib::RefPtr<Gtk::FileChooser> getObject();

protected:
	void initInstance();
	void initFileChooserInstance();
	
	void setFilters(Property * property, Any value);
	void setFilter(Property * property, Any value);
	void clearFilters();

	bool isActiveWidget(Gtk::Widget * widget);

	static void stopEmissionHandler(GtkWidget * widget, GtkWidget * prev, gpointer data);
	static void fixWidgetSignals(GtkWidget * widget);
	static void fixFileChooserDefault(GtkWidget * widget, gpointer data);
	static void fixFileChooserWidget(GtkWidget * widget, gpointer data);

};

class GtkFileChooserDialogView : public GtkDialogViewFinal, public GtkFileChooserViewBase {

public:
	GtkFileChooserDialogView();

	Glib::RefPtr<Gtk::FileChooser> getObject();
	
protected:
	Any createInstance();
	void initInstance();
	void initDesign();

};

class GtkFontSelectionDialogView : public GtkDialogViewFinal {

public:
	GtkFontSelectionDialogView();
	
protected:
	Any createInstance();
	void configureFontSelection(Property * property, PEntityView view);
	Any getFontSelection(Property * property);

};

class GtkStatusbarView : public GtkContainerViewBase, public TCreateInstance<Gtk::Statusbar> {

public:
	GtkStatusbarView();

};

class GtkActionGroupView : public GlibObjectView {

public:
	GtkActionGroupView();

protected:
	int getNextId();
	Any createInstance();
	void doneInstance();
	void setActions(Property * property, Any value);
	void clearActions();
	std::string getActionLabel(Property * property, PNode node);

};

class GtkActionView : public virtual GlibObjectView {

public:
	GtkActionView();

protected:
	Any createInstance();
	void doneInstance();
	Glib::ustring createName();
	void setLabel(Property * property, Any value);
	void setStockId(Property * property, Any value);

	void clearWidgets();
	void setWidgets(Property * property, Any value);

private:
	typedef std::list<PGtkWidget> Widgets;
	Widgets widgets;

};

class GtkToggleActionView : public GtkActionView {

public:
	GtkToggleActionView();

protected:
	Any createInstance();

};

class GtkRadioActionView : public GtkToggleActionView {

public:
	GtkRadioActionView();

protected:
	Any createInstance();
	void setActive(Property * property, Any value);

private:
	Glib::RefPtr<Gtk::RadioAction> dummy;

};

class GtkUIManagerView : public GlibObjectView {

public:
	GtkUIManagerView();

	Gtk::UIManager::ui_merge_id getMergeId(int index);

protected:
	Any createInstance();
	void setActionGroups(Property * property, Any value);

private:
	typedef std::vector<Gtk::UIManager::ui_merge_id> MergeIds;
	MergeIds mergeIds;

};

class UIDefinition : public Glib::Object {

public:
	UIDefinition();
	Glib::PropertyProxy<Glib::ustring> property_ui() { return propertyUi.get_proxy(); }

private:
	Glib::Property<Glib::ustring> propertyUi;

};

class UIDefinitionView : public TCreateInstance<UIDefinition> {

public:
	UIDefinitionView();

};

class UIElementView : public GtkContainerViewBase {

public:
	UIElementView();

protected:
	Any createInstance();
	virtual bool validWidget(Gtk::Widget * widget) = 0;
	virtual PGtkWidget createStub() = 0;

};

class GtkMenuBarView : public UIElementView {

public:
	using GlibObjectView::getObject;

	GtkMenuBarView();

protected:
	bool validWidget(Gtk::Widget * widget);
	PGtkWidget createStub();

};

class GtkToolbarView : public UIElementView {

public:
	using GlibObjectView::getObject;

	GtkToolbarView();

protected:
	bool validWidget(Gtk::Widget * widget);
	PGtkWidget createStub();
	void setToolbarStyleSet(Property * property, Any value);
	void setIconSizeSet(Property * property, Any value);

};

class RadioGroup : public Glib::Object {

public:
	RadioGroup();
	Gtk::RadioButtonGroup * getGroup() { return &group; }

private:
	Gtk::RadioButtonGroup group;

};

class RadioGroupView : public TCreateInstance<RadioGroup> {
};

class GtkMiscView : public GtkWidgetView {

public:
	GtkMiscView();

};

class GtkLabelView : public GtkMiscView, public TCreateInstance<Gtk::Label> {

public:
	using TCreateInstance<Gtk::Label>::getObject;

	GtkLabelView();

protected:
	void initDesign();

};

class GtkButtonViewBase : public virtual GtkWidgetView {

public:
	GtkButtonViewBase();

protected:
	void initInstance();
	void setSensitive(Property * property, Any value);
	void setUAA(bool uaa);
	bool getUAA();

};

class GtkButtonView : public GtkBinView, public GtkButtonViewBase, public TCreateInstance<Gtk::Button> {

public:
	using TCreateInstance<Gtk::Button>::getObject;

	GtkButtonView();

protected:
	enum ButtonState { bsNormal, bsActivatable, bsStock, bsContainer };

	void initInstance();

	void initDesign();
	void setupContainer();
	void onAsContainerSetup();
	void resetButton();
	void setState(ButtonState bs);

	void setUseActionAppearance(Property * property, Any value);
	void setStockId(Property * property, Any value);
	void setImage(Property * property, Any value);
	void setUseUnderline(Property * property, Any value);

	bool isActiveWidget(Gtk::Widget * widget);

};

class GtkToggleButtonView : public GtkButtonView {

public:
	GtkToggleButtonView();

protected:
	Any createInstance();

};

class GtkCheckButtonView : public GtkToggleButtonView {

public:
	GtkCheckButtonView();

protected:
	Any createInstance();

};

class GtkRadioButtonView : public GtkCheckButtonView {

public:
	using GlibObjectView::getObject;

	GtkRadioButtonView();

protected:
	Any createInstance();
	void setActive(Property * property, Any value);

private:
	Glib::RefPtr<Gtk::RadioButton> dummy;

};

class GtkScaleButtonView : public GtkContainerViewBase, public GtkButtonViewBase {

public:
	GtkScaleButtonView();

protected:
	Any createInstance();
	void setIcons(Property * property, Any value);

};

}

#endif
